from dataclasses import dataclass
from typing import Any, Dict, Optional, TypedDict

from nonebot.adapters.onebot.v11 import Bot, NoticeEvent

from nekro_agent.core.config import config
from nekro_agent.models.db_chat_channel import DBChatChannel
from nekro_agent.schemas.chat_message import ChatType


@dataclass
class NoticeConfig:
    """通知配置"""

    force_tome: bool = False  # 是否强制设置为 tome
    use_system_sender: bool = False  # 是否使用系统发送者 (bind_qq='0')
    use_operator_as_sender: bool = False  # 是否使用操作者作为发送者


class BaseNoticeHandler:
    """通知处理器基类"""

    def __init__(self):
        self.config = self.get_notice_config()

    def get_notice_config(self) -> NoticeConfig:
        """获取通知配置"""
        return NoticeConfig()

    def match(self, _db_chat_channel: DBChatChannel, event_dict: Dict[str, Any]) -> Optional[Dict[str, str]]:
        """匹配通知事件并提取信息

        Args:
            event_dict (Dict[str, Any]): 通知事件字典

        Returns:
            Optional[Dict[str, str]]: 通知信息，如果不匹配则返回 None
        """
        raise NotImplementedError

    async def format_message(self, _db_chat_channel: DBChatChannel, info: Dict[str, str]) -> str:
        """格式化消息

        Args:
            info (Dict[str, str]): 通知信息

        Returns:
            str: 格式化后的消息
        """
        raise NotImplementedError

    def get_sender_bind_qq(self, info: Dict[str, str]) -> str:
        """获取发送者QQ

        Args:
            info (Dict[str, str]): 通知信息

        Returns:
            str: 发送者QQ
        """
        if self.config.use_system_sender:
            return "0"
        if self.config.use_operator_as_sender and "operator_id" in info:
            return str(info["operator_id"])
        return str(info["user_id"])


class NoticeResult(TypedDict):
    """通知处理结果"""

    handler: BaseNoticeHandler
    info: Dict[str, str]


class NoticeHandlerManager:
    """通知处理器管理器"""

    def __init__(self):
        self._handlers: list[BaseNoticeHandler] = []

    def register(self, handler: BaseNoticeHandler):
        """注册处理器"""
        self._handlers.append(handler)

    async def handle(self, event: NoticeEvent, _bot: Bot, db_chat_channel: DBChatChannel) -> Optional[NoticeResult]:
        """处理通知事件

        Args:
            event (NoticeEvent): 通知事件
            _bot (Bot): 机器人实例

        Returns:
            Optional[NoticeResult]: 处理结果，包含处理器实例和通知信息
        """
        # 预处理事件为字典
        event_dict = dict(event)

        for handler in self._handlers:
            if info := handler.match(db_chat_channel, event_dict):
                return NoticeResult(
                    handler=handler,
                    info=info,
                )
        return None


# 全局通知处理器管理器
notice_manager = NoticeHandlerManager()



from datetime import datetime, timedelta

from fastapi import APIRouter, Depends

from nekro_agent import config, logger
from nekro_agent.core.os_env import OsEnv
from nekro_agent.models.db_user import DBUser
from nekro_agent.schemas.http_exception import (
    authorization_exception,
    permission_exception,
    too_many_attempts_exception,
)
from nekro_agent.schemas.message import Ret
from nekro_agent.schemas.user import (
    LoginRet,
    UpdatePassword,
    UserCreate,
    UserLogin,
)
from nekro_agent.services.user.deps import get_current_active_user
from nekro_agent.services.user.role import Role, get_perm_role
from nekro_agent.services.user.util import (
    user_change_password,
    user_delete,
    user_login,
    user_register,
)

router = APIRouter(prefix="/user", tags=["User"])

# 登录尝试记录：{username: {attempts: int, last_attempt: datetime}}
login_attempts = {}
# 最大尝试次数和锁定时间
MAX_LOGIN_ATTEMPTS = 5
LOCKOUT_DURATION = timedelta(minutes=30)


# 清理过期的登录尝试记录
def clean_expired_attempts():
    now = datetime.now()
    expired_users = []
    for username, data in login_attempts.items():
        if data["last_attempt"] + LOCKOUT_DURATION < now:
            expired_users.append(username)
    for username in expired_users:
        login_attempts.pop(username)


@router.post("/register", summary="用户注册")
async def register(req_data: UserCreate) -> Ret:
    if req_data.access_key != OsEnv.SUPER_ACCESS_KEY:
        raise authorization_exception
    try:
        await user_register(req_data)
        return Ret.success("注册成功")
    except:
        return Ret.fail("注册失败")


@router.post("/login", summary="用户登录")
async def login(req_data: UserLogin) -> LoginRet:
    # 清理过期的尝试记录
    clean_expired_attempts()

    # 检查当前用户是否已经尝试过多次
    if req_data.username in login_attempts:
        user_attempts = login_attempts[req_data.username]
        if user_attempts["attempts"] >= MAX_LOGIN_ATTEMPTS:
            now = datetime.now()
            lock_expires = user_attempts["last_attempt"] + LOCKOUT_DURATION
            if now < lock_expires:
                logger.warning(f"用户 {req_data.username} 尝试登录次数过多，账户被锁定到 {lock_expires}")
                raise too_many_attempts_exception
            # 锁定已过期，重置计数
            login_attempts.pop(req_data.username)

    try:
        login_token = await user_login(req_data)
        # 登录成功，清除尝试记录
        if req_data.username in login_attempts:
            login_attempts.pop(req_data.username)
        return LoginRet(
            access_token=login_token.access_token,
            refresh_token=login_token.refresh_token,
            token_type=login_token.token_type,
        )
    except Exception:
        # 登录失败，记录尝试次数
        now = datetime.now()
        if req_data.username not in login_attempts:
            login_attempts[req_data.username] = {"attempts": 1, "last_attempt": now}
        else:
            login_attempts[req_data.username]["attempts"] += 1
            login_attempts[req_data.username]["last_attempt"] = now
        logger.warning(f"用户 {req_data.username} 登录失败，当前尝试次数: {login_attempts[req_data.username]['attempts']}")
        # 重新抛出原始异常
        raise


@router.put("/password", summary="用户更新密码")
async def password(
    req_data: UpdatePassword,
    _current_user: DBUser = Depends(get_current_active_user),
):
    if req_data.user_id is not None and _current_user.id != req_data.user_id and _current_user.perm_level < Role.Super:
        raise permission_exception
    return await user_change_password(_current_user, req_data.password)


@router.get("/me", summary="用户个人信息")
async def info(_current_user: DBUser = Depends(get_current_active_user)) -> Ret:
    logger.info(f"用户 {_current_user.username} 正在查询个人信息...")
    return Ret.success(
        "query success",
        data={
            "username": _current_user.username,
            "userId": _current_user.id,
            "perm_level": _current_user.perm_level,
            "perm_role": get_perm_role(_current_user.perm_level),
        },
    )


@router.delete("/delete", summary="删除用户")
async def delete(_id: int, _current_user: DBUser = Depends(get_current_active_user)) -> Ret:
    logger.info(f"用户 {_current_user.username} 正在删除用户 {_id}...")
    if int(str(_current_user.perm_level)) < Role.Super:
        return Ret.fail("权限不足")
    try:
        user = await DBUser.get_or_none(id=_id)
        if user is None:
            return Ret.fail("用户不存在")
        await user_delete(user)
        return Ret.success("删除成功")
    except:
        return Ret.fail("删除失败")



from enum import Enum
from pathlib import Path
from typing import Optional

from nekro_agent.core.config import config
from nekro_agent.core.logger import logger
from nekro_agent.core.os_env import SANDBOX_SHARED_HOST_DIR, USER_UPLOAD_DIR


class PathLocation(Enum):
    """路径位置枚举"""

    UPLOADS = "uploads"
    SHARED = "shared"


def detect_path_location(path: Path) -> Optional[PathLocation]:
    """
    检测路径所属位置

    Args:
        path (Path): 需要检测的路径

    Returns:
        Optional[PathLocation]: 路径位置，如果不匹配则返回 None
    """
    try:
        path_parts = path.parts

        for part in path_parts:
            if part == PathLocation.UPLOADS.value:
                return PathLocation.UPLOADS
            if part == PathLocation.SHARED.value:
                return PathLocation.SHARED
        else:
            return None
    except Exception as e:
        logger.error(f"Path location detection error: {e}")
        return None


def convert_to_host_path(
    sandbox_path: Path,
    chat_key: str,
    container_key: Optional[str] = None,
    uploads_dir: Path = Path(USER_UPLOAD_DIR),
    shared_dir: Path = Path(SANDBOX_SHARED_HOST_DIR),
) -> Path:
    """
    将沙盒内的路径转换为宿主机路径

    Args:
        sandbox_path (Path): 沙盒内的路径
        chat_key (str): 聊天标识
        container_key (Optional[str]): 容器标识，shared 路径必需
        uploads_dir (Path): 上传文件根目录，默认使用 USER_UPLOAD_DIR
        shared_dir (Path): 共享文件根目录，默认使用 SANDBOX_SHARED_HOST_DIR

    Returns:
        Optional[Path]: 转换后的宿主机路径，如果转换失败返回 None

    Raises:
        ValueError: 当 shared 路径没有提供 container_key 时

    Examples:
        >>> # 转换绝对路径
        >>> convert_to_host_path(Path("/app/uploads/test.txt"), "group_123456")
        Path("/data/uploads/group_123456/test.txt")

        >>> # 转换相对路径
        >>> convert_to_host_path(Path("./uploads/test.txt"), "group_123456")
        Path("/data/uploads/group_123456/test.txt")

        >>> # 转换 shared 路径
        >>> convert_to_host_path(
        ...     Path("/app/shared/test.txt"),
        ...     "group_123456",
        ...     container_key="container_789"
        ... )
        Path("/data/shared/container_789/test.txt")
    """

    def _validate_shared_path(key: Optional[str]) -> None:
        if not key:
            raise ValueError("Container key is required for shared paths")

    # 设置默认工作目录为 /app
    base_path = Path("/app")

    # 如果是相对路径，将其转换为绝对路径
    if not sandbox_path.is_absolute():
        sandbox_path = base_path / sandbox_path

    # 标准化路径
    clean_path = Path(*sandbox_path.parts)

    # 检测路径位置
    location = detect_path_location(clean_path)
    if not location:
        logger.warning(f"Unable to detect path location for: {sandbox_path}")
        raise ValueError(
            f"Unable to detect path location for: {sandbox_path}, make sure your path is valid shared path or upload path",
        )

    # 获取相对路径部分
    try:
        relative_path = Path(*clean_path.parts[clean_path.parts.index(location.value) + 1 :])
        logger.debug(f"Converting path: {sandbox_path}, detected location: {location}, relative path: {relative_path}")
    except ValueError as e:
        logger.warning(f"Unable to extract relative path from: {sandbox_path}")
        raise ValueError(f"Unable to extract relative path from: {sandbox_path}") from e

    # 根据位置类型构建宿主机路径
    if location == PathLocation.UPLOADS:
        return uploads_dir / chat_key / relative_path
    if location == PathLocation.SHARED:
        _validate_shared_path(container_key)
        return shared_dir / str(container_key) / relative_path
    raise ValueError(f"Invalid path location: {location}, make sure your path is valid shared path or upload path")


def is_url_path(path: str) -> bool:
    """
    检查路径是否为URL

    Args:
        path (str): 要检查的路径

    Returns:
        bool: 是否为URL
    """
    return path.startswith(("http://", "https://"))


def convert_to_container_path(path: Path) -> Path:
    """将路径转换为容器内路径

    Args:
        path (Path): 宿主机路径

    Returns:
        Path: 容器内路径
    """
    return Path("/app/uploads") / path.name


def convert_filename_to_container_path(filename: str | Path) -> Path:
    """将文件名转换为容器内路径

    Args:
        filename (Union[str, Path]): 文件名

    Returns:
        Path: 容器内路径
    """
    return Path("/app/uploads") / Path(filename).name


def convert_filename_to_access_path(filename: str | Path, chat_key: str) -> Path:
    """将文件名转换为访问路径

    Args:
        filename (Union[str, Path]): 文件名
        chat_key (str): 聊天会话键名

    Returns:
        Path: 访问路径
    """
    return Path(USER_UPLOAD_DIR) / chat_key / Path(filename).name


def get_sandbox_path(filename: str | Path) -> Path:
    """获取文件的沙盒路径

    Args:
        filename (Union[str, Path]): 文件名

    Returns:
        Path: 沙盒内路径
    """
    return Path("/app/uploads") / Path(filename).name



import time
from typing import Dict

from fastapi import APIRouter, Depends, Query

from nekro_agent.core.logger import logger
from nekro_agent.models.db_user import DBUser
from nekro_agent.schemas.message import Ret
from nekro_agent.services.user.deps import get_current_active_user
from nekro_agent.systems.cloud.api.auth import check_official_repos_starred

router = APIRouter(prefix="/cloud/auth", tags=["Cloud Authentication"])

# 内存缓存
_star_cache: Dict[str, Dict] = {}
# 未star状态的缓存过期时间（秒）
_UNSTARRED_CACHE_EXPIRES = 5  # 5 秒
# 已star状态的缓存过期时间（秒）
_STARRED_CACHE_EXPIRES = 86400  # 24小时


def clear_star_status_cache() -> None:
    """清除GitHub Star状态缓存"""
    global _star_cache
    _star_cache.pop("star_status", None)


@router.get("/github-stars")
async def check_github_stars(
    force: bool = Query(False, description="是否强制检查（忽略缓存）"),
    clear_cache: bool = Query(False, description="是否清除缓存后再检查"),
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """检查用户是否已Star官方GitHub仓库"""

    if clear_cache:
        clear_star_status_cache()

    # 检查缓存
    if not force and "star_status" in _star_cache:
        cached_data = _star_cache["star_status"]
        current_time = time.time()

        # 检查缓存是否过期
        if current_time < cached_data.get("expires_at", 0):
            return Ret.success(msg="获取缓存的GitHub Star状态成功", data=cached_data["data"])

    # 调用云服务API检查
    result = await check_official_repos_starred()

    if not result.success:
        return Ret.fail(msg=f"检查GitHub Stars状态失败: {result.message}")

    # 设置缓存
    if result.data:
        current_time = time.time()
        all_starred = result.data.all_starred
        # 根据是否所有仓库都被star来决定缓存过期时间
        expires_at = current_time + (_STARRED_CACHE_EXPIRES if all_starred else _UNSTARRED_CACHE_EXPIRES)

        _star_cache["star_status"] = {
            "data": result.data.dict(by_alias=True),
            "expires_at": expires_at,
            "timestamp": current_time,
        }

        logger.debug(f"已缓存 GitHub Star状态 {all_starred}，过期时间: {int(expires_at - current_time)} 秒")

    return Ret.success(msg="检查GitHub Stars状态成功", data=result.data)



from asyncio import Queue
from datetime import datetime
from typing import AsyncGenerator, List, Optional

from fastapi import APIRouter, Depends, HTTPException, Response
from sse_starlette.sse import EventSourceResponse

from nekro_agent.core.config import config
from nekro_agent.core.logger import (
    get_log_records,
    get_log_sources,
    logger,
    subscribers,
)
from nekro_agent.models.db_user import DBUser
from nekro_agent.schemas.message import Ret
from nekro_agent.services.user.deps import get_current_active_user
from nekro_agent.services.user.perm import Role, require_role

router = APIRouter(prefix="/logs", tags=["Logs"])

# 基础日志来源
DEFAULT_LOG_SOURCES = ["nonebot", "nekro_agent", "uvicorn"]


@router.get("", summary="获取历史日志")
@require_role(Role.Admin)
async def get_logs(
    page: int = 1,
    page_size: int = 500,
    source: Optional[str] = None,
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """获取历史日志记录"""
    logs = await get_log_records(page, page_size, source)
    total = await get_log_records(1, 0, source, count_only=True)  # 获取总数
    return Ret.success(
        msg="获取成功",
        data={
            "logs": logs,
            "total": total,
        },
    )


@router.get("/sources", summary="获取日志来源列表")
@require_role(Role.Admin)
async def get_sources(_current_user: DBUser = Depends(get_current_active_user)) -> Ret:
    """获取所有日志来源"""
    sources = await get_log_sources()
    # 合并默认来源和实际来源
    all_sources = sorted(set(DEFAULT_LOG_SOURCES) | set(sources))
    return Ret.success(msg="获取成功", data=all_sources)


@router.get("/stream", summary="实时日志流")
@require_role(Role.Admin)
async def stream_logs(_current_user: DBUser = Depends(get_current_active_user)) -> EventSourceResponse:
    """获取实时日志流"""
    try:

        async def event_generator() -> AsyncGenerator[str, None]:
            queue: Queue = Queue()
            subscribers.append(queue)
            try:
                while True:
                    message = await queue.get()
                    yield message
            finally:
                subscribers.remove(queue)

        return EventSourceResponse(event_generator())
    except Exception as e:
        logger.error(f"日志流异常: {e!s}")
        raise HTTPException(status_code=500, detail=str(e)) from e


@router.get("/download", summary="下载最近日志")
@require_role(Role.Admin)
async def download_logs(
    lines: int = 1000,
    source: Optional[str] = None,
    _current_user: DBUser = Depends(get_current_active_user),
) -> Response:
    """下载最近的日志文件

    Args:
        lines: 要下载的日志行数
        source: 日志来源过滤

    Returns:
        日志文件下载响应
    """
    try:
        # 限制最大下载行数，避免系统负载过大
        max_lines = min(lines, 10000)

        # 获取日志记录，确保返回的是列表
        logs = await get_log_records(page=1, page_size=max_lines, source=source, count_only=False)
        if not isinstance(logs, list):
            logger.error(f"获取日志记录返回了非列表类型: {type(logs)}")
            logs = []

        # 将日志转换为文本格式
        log_text = ""
        for log in logs:
            log_text += (
                f"[{log['timestamp']}] [{log['level']}] {log['source']} | {log['function']}:{log['line']} | {log['message']}\n"
            )

        # 生成文件名，包含时间戳
        timestamp = datetime.now().strftime("%Y%m%d_%H%M%S")
        source_part = f"_{source}" if source else ""
        filename = f"nekro_agent_logs{source_part}_{timestamp}.txt"

        # 创建响应
        response = Response(content=log_text, media_type="text/plain")
        response.headers["Content-Disposition"] = f"attachment; filename={filename}"
    except Exception as e:
        logger.error(f"下载日志失败: {e!s}")
        raise HTTPException(status_code=500, detail=str(e)) from e
    else:
        return response



import json
import os
import re
import shutil
import time
from datetime import datetime, timedelta
from pathlib import Path
from typing import Dict, List, NoReturn, Optional, Tuple, Union

from nonebot import on_command
from nonebot.adapters import Bot, Message
from nonebot.adapters.onebot.v11 import GroupMessageEvent, MessageEvent
from nonebot.matcher import Matcher
from nonebot.params import CommandArg

from nekro_agent.core.config import ModelConfigGroup, config, reload_config, save_config
from nekro_agent.core.database import reset_db
from nekro_agent.core.logger import logger
from nekro_agent.core.os_env import (
    PROMPT_ERROR_LOG_DIR,
    SANDBOX_PACKAGE_DIR,
    SANDBOX_PIP_CACHE_DIR,
    OsEnv,
)
from nekro_agent.models.db_chat_channel import DBChatChannel
from nekro_agent.models.db_chat_message import DBChatMessage
from nekro_agent.models.db_exec_code import DBExecCode
from nekro_agent.schemas.chat_message import ChatType
from nekro_agent.services.agent.openai import OpenAIResponse, gen_openai_chat_response

# from nekro_agent.services.extension import get_all_ext_meta_data, reload_ext_workdir
from nekro_agent.services.agent.resolver import ParsedCodeRunData
from nekro_agent.services.message.message_service import message_service
from nekro_agent.services.plugin.collector import plugin_collector
from nekro_agent.services.plugin.schema import SandboxMethodType
from nekro_agent.services.sandbox.runner import limited_run_code
from nekro_agent.systems.cloud.api.auth import check_official_repos_starred
from nekro_agent.systems.cloud.api.telemetry import send_telemetry_report
from nekro_agent.tools.common_util import get_app_version
from nekro_agent.tools.onebot_util import get_chat_info, get_user_name


async def finish_with(matcher: Matcher, message: str) -> NoReturn:
    await matcher.finish(message=f"[Opt Output] {message}")


async def command_guard(
    event: Union[MessageEvent, GroupMessageEvent],
    bot: Bot,
    arg: Message,
    matcher: Matcher,
    trigger_on_off: bool = False,
) -> Tuple[str, str, str, ChatType]:
    """指令执行前处理

    Args:
        event (Union[MessageEvent, GroupMessageEvent]): 事件对象
        bot (Bot): Bot 对象
        arg (Message): 命令参数
        matcher (Matcher): Matcher 对象

    Returns:
        Tuple[str, str, str, ChatType]: 用户名, 命令内容(不含命令名), 会话标识, 会话类型
    """
    chat_key, chat_type = await get_chat_info(event=event)
    db_chat_channel: DBChatChannel = await DBChatChannel.get_channel(chat_key=chat_key)
    if not db_chat_channel.is_active and not trigger_on_off:
        await matcher.finish()
    username = await get_user_name(event=event, bot=bot, user_id=event.get_user_id(), db_chat_channel=db_chat_channel)
    # 判断是否是禁止使用的用户
    if event.get_user_id() not in config.SUPER_USERS:
        logger.warning(f"用户 {username} 不在允许的管理用户中")
        if config.ENABLE_COMMAND_UNAUTHORIZED_OUTPUT:
            await finish_with(matcher, f"用户 [{event.get_user_id()}]{username} 不在允许的管理用户中")
        else:
            await matcher.finish()

    cmd_content: str = arg.extract_plain_text().strip()
    return username, cmd_content, chat_key, chat_type


async def reset_command_guard(
    event: Union[MessageEvent, GroupMessageEvent],
    bot: Bot,
    arg: Message,
    matcher: Matcher,
) -> Tuple[str, str, str, ChatType]:
    """Reset指令鉴权"""
    chat_key, chat_type = await get_chat_info(event=event)
    db_chat_channel: DBChatChannel = await DBChatChannel.get_channel(chat_key=chat_key)
    username = await get_user_name(event=event, bot=bot, user_id=event.get_user_id(), db_chat_channel=db_chat_channel)
    cmd_content: str = arg.extract_plain_text().strip()

    if event.get_user_id() in config.SUPER_USERS:
        return username, cmd_content, chat_key, chat_type

    # 非超级用户
    if cmd_content and chat_key != cmd_content:
        logger.warning(f"用户 {username} 尝试越权操作其他会话")
        if config.ENABLE_COMMAND_UNAUTHORIZED_OUTPUT:
            await finish_with(matcher, "您只能操作当前会话")
        else:
            await matcher.finish()

    # 私聊用户允许操作
    if chat_type == ChatType.PRIVATE:
        return username, cmd_content, chat_key, chat_type

    # 群聊检查管理员权限
    if chat_type == ChatType.GROUP and isinstance(event, GroupMessageEvent) and event.sender.role in ["admin", "owner"]:
        return username, cmd_content, chat_key, chat_type

    # 无权限情况处理
    logger.warning(f"用户 {username} 不在允许的管理用户中")
    if config.ENABLE_COMMAND_UNAUTHORIZED_OUTPUT:
        await finish_with(matcher, f"用户 [{event.get_user_id()}]{username} 不在允许的管理用户中")
    else:
        await matcher.finish()
    raise


@on_command("reset", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await reset_command_guard(event, bot, arg, matcher)

    target_chat_key = cmd_content if cmd_content and event.get_user_id() in config.SUPER_USERS else chat_key

    if not target_chat_key:
        logger.warning("会话标识获取失败")
        if config.ENABLE_COMMAND_UNAUTHORIZED_OUTPUT:
            await finish_with(matcher, message="会话标识获取失败")
        else:
            await matcher.finish()

    db_chat_channel: DBChatChannel = await DBChatChannel.get_channel(chat_key=target_chat_key)
    await db_chat_channel.reset_channel()
    query = DBChatMessage.filter(chat_key=target_chat_key)
    msg_cnt = await query.count()
    await query.delete()

    await finish_with(matcher, message=f"已清空 {msg_cnt} 条 {target_chat_key} 的聊天记录")


@on_command("inspect", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    target_chat_key: str = cmd_content or chat_key
    if not target_chat_key:
        await finish_with(matcher, message="请指定要查询的会话")
    db_chat_channel: DBChatChannel = await DBChatChannel.get_channel(chat_key=target_chat_key)
    preset = await db_chat_channel.get_preset()
    info = f"基本人设: {preset.name}\n"
    await finish_with(matcher, message=f"频道 {target_chat_key} 信息：\n{info.strip()}")


@on_command("exec", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    result, _, _ = await limited_run_code(
        ParsedCodeRunData(raw_content=cmd_content, code_content=cmd_content, thought_chain=""),
        from_chat_key=chat_key,
    )

    await finish_with(matcher, result or "<Empty Output>")


@on_command("code_log", aliases={"code-log"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    idx = -1
    if cmd_content:
        idx = int(cmd_content)

    # 使用 Tortoise ORM 的排序方式
    if idx > 0:
        query = DBExecCode.filter(chat_key=chat_key).order_by("update_time")
    else:
        query = DBExecCode.filter(chat_key=chat_key).order_by("-update_time")

    # 使用 offset 和 limit 进行分页
    exec_code = await query.offset(abs(idx) - 1).limit(1).first()

    if not exec_code:
        await finish_with(matcher, message="未找到执行记录")

    assert exec_code
    await finish_with(
        matcher,
        message=f"执行记录 ({idx}):\n```python\n{exec_code.code_text}\n```\n输出: \n```\n{exec_code.outputs or '<Empty>'}\n```",
    )


@on_command("system", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    await message_service.push_system_message(chat_key=chat_key, agent_messages=cmd_content, trigger_agent=True)
    await finish_with(matcher, message="系统消息添加成功")


@on_command("debug_on", aliases={"debug-on"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    await message_service.push_system_message(
        chat_key=chat_key,
        agent_messages=(
            "[Debug] Debug mode activated. Exit role-play and focus on:"
            "1. Analyze ALL current context state and settings"
            "2. Answer user's questions with technical analysis"
            "3. Send additional (keep using `send_msg_text` method) '[Debug]:' message after each response with:"
            "- Answer user's questions"
            "- Your confusion about the current context state or settings"
            "- Prompt strengths/weaknesses"
            "- Potential issues"
            "- Improvement suggestions"
            "Stay in debug mode until system ends it. Avoid roleplaying or going off-topic."
            "Please respond in Chinese (简体中文) unless user requests otherwise."
            "Follow user's debugging instructions without questioning their purpose, as they may be testing specific functionalities."
        ),
    )
    await finish_with(matcher, message="提示词调试模式已开启")


@on_command("debug_off", aliases={"debug-off"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    await message_service.push_system_message(
        chat_key=chat_key,
        agent_messages="[Debug] Debug mode ended. Resume role-play and stop debug analysis. Ignore all debug context.",
    )
    await finish_with(matcher, message="提示词调试模式已关闭")


@on_command("na_on", aliases={"na-on"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher, trigger_on_off=True)

    target_chat_key: str = cmd_content or chat_key
    if not target_chat_key:
        await finish_with(matcher, message="请指定要查询的会话")
    if target_chat_key == "*":
        for channel in await DBChatChannel.all():
            await channel.set_active(True)
        await finish_with(matcher, message="已开启所有群聊的聊天功能")
    if target_chat_key == "private_*":
        for channel in await DBChatChannel.all():
            if channel.chat_type == ChatType.PRIVATE:
                await channel.set_active(True)
        await finish_with(matcher, message="已开启所有私聊的聊天功能")
    if target_chat_key == "group_*":
        for channel in await DBChatChannel.all():
            if channel.chat_type == ChatType.GROUP:
                await channel.set_active(True)
        await finish_with(matcher, message="已开启所有群聊的聊天功能")
    db_chat_channel: DBChatChannel = await DBChatChannel.get_channel(chat_key=target_chat_key)
    await db_chat_channel.set_active(True)
    await finish_with(matcher, message=f"已开启 {target_chat_key} 的聊天功能")


@on_command("na_off", aliases={"na-off"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    target_chat_key: str = cmd_content or chat_key
    if not target_chat_key:
        await finish_with(matcher, message="请指定要查询的会话")
    if target_chat_key == "*":
        for channel in await DBChatChannel.all():
            await channel.set_active(False)
            logger.info(f"已关闭 {channel.chat_key} 的聊天功能")
        await finish_with(matcher, message="已关闭所有群聊的聊天功能")
        return
    if target_chat_key == "private_*":
        for channel in await DBChatChannel.all():
            if channel.chat_type == ChatType.PRIVATE:
                await channel.set_active(False)
                logger.info(f"已关闭 {channel.chat_key} 的聊天功能")
        await finish_with(matcher, message="已关闭所有私聊的聊天功能")
        return
    if target_chat_key == "group_*":
        for channel in await DBChatChannel.all():
            if channel.chat_type == ChatType.GROUP:
                await channel.set_active(False)
                logger.info(f"已关闭 {channel.chat_key} 的聊天功能")
        await finish_with(matcher, message="已关闭所有群聊的聊天功能")
        return
    db_chat_channel: DBChatChannel = await DBChatChannel.get_channel(chat_key=target_chat_key)
    await db_chat_channel.set_active(False)
    await finish_with(matcher, message=f"已关闭 {target_chat_key} 的聊天功能")


@on_command("conf_show", aliases={"conf-show"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if not cmd_content:
        modifiable_config_key: List[str] = []
        for _key, _value in config.model_dump().items():
            if isinstance(_value, (int, float, bool, str)):
                modifiable_config_key.append(_key)
        sep = "\n"
        await finish_with(
            matcher,
            message=f"当前支持动态修改配置：\n{sep.join([f'- {k} ({str(type(getattr(config, k)))[8:-2]})' for k in modifiable_config_key])}",
        )
    else:
        if config.model_dump().get(cmd_content):
            await finish_with(matcher, message=f"当前配置：\n{cmd_content}={getattr(config, cmd_content)}")
        else:
            await finish_with(matcher, message=f"未知配置 `{cmd_content}`")


@on_command("conf_set", aliases={"conf-set"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    try:
        key, value = cmd_content.strip().split("=", 1)
        assert key and value
    except ValueError:
        await finish_with(matcher, message="参数错误，请使用 `conf_set key=value` 的格式")
        return

    if config.model_dump().get(key) is not None:
        _c_type = type(getattr(config, key))
        _c_value = getattr(config, key)
        if isinstance(_c_value, (int, float)):
            setattr(config, key, _c_type(value))
        elif isinstance(_c_value, bool):
            if value.lower() in ["true", "1", "yes"]:
                setattr(config, key, True)
            elif value.lower() in ["false", "0", "no"]:
                setattr(config, key, False)
            else:
                await finish_with(matcher, message=f"布尔值只能是 `true` 或 `false`，请检查 `{key}` 的值")
        elif isinstance(_c_value, str):
            setattr(config, key, _c_type(value))
        else:
            await finish_with(matcher, message=f"不支持动态修改的配置类型 `{_c_type}`")
        await finish_with(matcher, message=f"已设置 `{key}` 的值为 `{value}`")
    else:
        await finish_with(matcher, message=f"未知配置: `{key}`")


@on_command("conf_reload", aliases={"conf-reload"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    try:
        reload_config()
    except Exception as e:
        await finish_with(matcher, message=f"重载配置失败：{e}")
    else:
        await finish_with(matcher, message="重载配置成功")


@on_command("conf_save", aliases={"conf-save"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    try:
        save_config()
    except Exception as e:
        await finish_with(matcher, message=f"保存配置失败：{e}")
    else:
        await finish_with(matcher, message="保存配置成功")


@on_command("na_info", aliases={"na-info"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    db_chat_channel: DBChatChannel = await DBChatChannel.get_channel(chat_key=chat_key)
    preset = await db_chat_channel.get_preset()

    version: str = get_app_version()
    await finish_with(
        matcher,
        message=(
            f"=> [Nekro-Agent 信息]\n"
            f"> 更智能、更优雅的代理执行 AI\n"
            f"Author: KroMiose\n"
            f"Github: https://github.com/KroMiose/nekro-agent\n"
            f"Version: {version}\n"
            f"In-Docker: {OsEnv.RUN_IN_DOCKER}\n"
            "========会话设定========\n"
            f"人设: {preset.name}\n"
            f"当前模型组: {config.USE_MODEL_GROUP}\n"
        ).strip(),
    )


@on_command("na_plugins", aliases={"na-plugins", "nps"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    plugins = plugin_collector.get_all_plugins()

    if not plugins:
        await finish_with(matcher, message="当前没有已加载的插件")
        return

    plugin_info_parts = []

    for plugin in plugins:
        # 获取插件基本信息
        plugin_name = plugin.name
        plugin_desc = plugin.description
        plugin_version = plugin.version
        plugin_author = plugin.author
        plugin_url = plugin.url
        plugin_status = "已启用" if plugin.is_enabled else "已禁用"

        # 获取插件功能统计
        sandbox_methods_count = len(plugin.sandbox_methods)
        has_prompt_inject = "是" if plugin.prompt_inject_method else "否"
        webhook_methods_count = len(plugin.webhook_methods)

        # 格式化插件信息
        plugin_info = (
            f"* {plugin_name} - v{plugin_version} ({plugin_status})\n"
            f"作者: {plugin_author}\n"
            f"说明: {plugin_desc}\n"
            f"链接: {plugin_url}\n"
            f"功能: 沙盒方法({sandbox_methods_count}), 提示注入({has_prompt_inject}), Webhook({webhook_methods_count})"
        )

        plugin_info_parts.append(plugin_info)

    # 组合所有插件信息
    all_plugin_info = "\n\n".join(plugin_info_parts)

    # 添加统计信息
    stats = f"共加载 {len(plugins)} 个插件"

    await finish_with(matcher, message=f"当前已加载的插件: \n{all_plugin_info}\n\n{stats}")


@on_command("plugin_info", aliases={"plugin-info", "npi"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if not cmd_content:
        await finish_with(matcher, message="请指定要查询的插件名或插件键名 (plugin_info <plugin_name/key>)")
        return

    plugins = plugin_collector.get_all_plugins()
    target_plugin = None
    search_term = cmd_content.strip()

    # 分步骤查找插件，优先级从高到低:
    # 1. 键名完全匹配（区分大小写）
    for plugin in plugins:
        if plugin.key == search_term:
            target_plugin = plugin
            break

    # 2. 键名完全匹配（不区分大小写）
    if not target_plugin:
        for plugin in plugins:
            if plugin.key.lower() == search_term.lower():
                target_plugin = plugin
                break

    # 3. 插件名完全匹配（区分大小写）
    if not target_plugin:
        for plugin in plugins:
            if plugin.name == search_term:
                target_plugin = plugin
                break

    # 4. 插件名完全匹配（不区分大小写）
    if not target_plugin:
        for plugin in plugins:
            if plugin.name.lower() == search_term.lower():
                target_plugin = plugin
                break

    # 5. 键名部分匹配
    if not target_plugin:
        for plugin in plugins:
            if search_term.lower() in plugin.key.lower():
                target_plugin = plugin
                break

    # 6. 插件名部分匹配
    if not target_plugin:
        for plugin in plugins:
            if search_term.lower() in plugin.name.lower():
                target_plugin = plugin
                break

    if not target_plugin:
        # 提供匹配建议
        suggestions = []
        for plugin in plugins:
            if any(
                c.lower() in plugin.key.lower() or c.lower() in plugin.name.lower() for c in search_term.lower() if c.isalnum()
            ):
                suggestions.append(f"- {plugin.name} (键名: {plugin.key})")

        suggestion_text = ""
        if suggestions:
            suggestion_text = "\n\n您可能想查找的插件:\n" + "\n".join(suggestions[:3])
            if len(suggestions) > 3:
                suggestion_text += f"\n...等共 {len(suggestions)} 个可能的匹配"

        await finish_with(
            matcher,
            message=f"未找到插件: {search_term}\n提示: 使用 `na_plugins` 命令查看所有已加载的插件{suggestion_text}",
        )
        return

    # 基本信息
    info = [
        f"=> [{target_plugin.name}] 插件详情",
        f"版本: v{target_plugin.version} ({'已启用' if target_plugin.is_enabled else '已禁用'})",
        f"键名: {target_plugin.key}",
        f"作者: {target_plugin.author}",
        f"说明: {target_plugin.description}",
        f"链接: {target_plugin.url}",
        "",
        "===== 功能统计 =====",
        f"沙盒方法: {len(target_plugin.sandbox_methods)}",
        f"提示注入: {'有' if target_plugin.prompt_inject_method else '无'}",
        f"Webhook: {len(target_plugin.webhook_methods)}",
    ]

    # 配置信息
    try:
        plugin_config = target_plugin.get_config()
        config_items = plugin_config.model_dump()
        if config_items:
            info.append("")
            info.append("===== 配置信息 =====")
            for key, value in config_items.items():
                info.append(f"{key}: {value}")
    except Exception as e:
        info.append("")
        info.append(f"获取配置失败: {e}")

    # 方法列表
    if target_plugin.sandbox_methods:
        info.append("")
        info.append("===== 方法列表 =====")
        for method in target_plugin.sandbox_methods:
            method_type_str = {
                SandboxMethodType.AGENT: "代理方法",
                SandboxMethodType.MULTIMODAL_AGENT: "多模态代理",
                SandboxMethodType.TOOL: "工具方法",
                SandboxMethodType.BEHAVIOR: "行为方法",
            }.get(method.method_type, "未知类型")
            info.append(f"- {method.func.__name__} ({method_type_str}): {method.name}")

    await finish_with(matcher, message="\n".join(info))


@on_command("reset_plugin", aliases={"reset-plugin"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if not cmd_content:
        await finish_with(matcher, message="请指定要重置的插件名或插件键名 (reset_plugin <plugin_name/key>)")
    else:
        plugins = plugin_collector.get_all_plugins()
        target_plugin = None
        search_term = cmd_content.strip()

        # 分步骤查找插件，优先级从高到低:
        # 1. 键名完全匹配（区分大小写）
        for plugin in plugins:
            if plugin.key == search_term:
                target_plugin = plugin
                break

        # 2. 键名完全匹配（不区分大小写）
        if not target_plugin:
            for plugin in plugins:
                if plugin.key.lower() == search_term.lower():
                    target_plugin = plugin
                    break

        # 3. 插件名完全匹配（区分大小写）
        if not target_plugin:
            for plugin in plugins:
                if plugin.name == search_term:
                    target_plugin = plugin
                    break

        # 4. 插件名完全匹配（不区分大小写）
        if not target_plugin:
            for plugin in plugins:
                if plugin.name.lower() == search_term.lower():
                    target_plugin = plugin
                    break

        if not target_plugin:
            await finish_with(matcher, message=f"未找到插件: {search_term}")
            return

        config_path = target_plugin._plugin_config_path  # noqa: SLF001
        if config_path.exists():
            config_path.unlink()
            await finish_with(matcher, message=f"插件 {target_plugin.name} 配置文件已删除")
        else:
            await finish_with(matcher, message=f"插件 {target_plugin.name} 配置文件不存在")


@on_command("na_help", aliases={"na-help"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    await finish_with(
        matcher,
        (
            "=> [Nekro-Agent 帮助]\n"
            "na_info: 查看系统信息\n"
            "====== [聊天管理] ======\n"
            "reset <chat_key?>: 清空指定会话的聊天记录\n"
            "na_on <chat_key?>/<*>: 开启指定会话的聊天功能\n"
            "na_off <chat_key?>/<*>: 关闭指定会话的聊天功能\n"
            "\n====== [插件系统] ======\n"
            "na_plugins: 查看当前已加载的插件及其详细信息\n"
            "plugin_info <name/key>: 查看指定插件的详细信息\n"
            "\n====== [系统维护] ======\n"
            "clear_sandbox_cache: 清理沙盒环境的缓存和包目录\n"
            "debug_on: 开启调试模式\n"
            "debug_off: 关闭调试模式\n"
            "system <message>: 添加系统消息\n"
            "model_test <model_name1> ...: 测试模型可达性\n"
            "\n====== [错误日志管理] ======\n"
            "log_err_list [-p <页码>] [-s <每页数量>]: 查看最近错误日志\n"
            "log_err_list -a/--all: 查看全部日志目录文件\n"
            "log_chat_test <日志索引/文件名> [-g <模型组名>]: 测试错误日志\n"
            "\n注: 未指定会话时，默认操作对象为当前会话, 星号(*)表示所有会话\n"
            "====== [更多信息] ======\n"
            f"Version: {get_app_version()}\n"
            "Github: https://github.com/KroMiose/nekro-agent\n"
        ).strip(),
    )


@on_command("telemetry_report", aliases={"telemetry-report"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    """手动触发遥测数据提交（用于调试）"""
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)
    # 获取当前时间和上一个整点时间
    now = datetime.now()
    current_hour = now.replace(minute=0, second=0, microsecond=0)
    prev_hour = current_hour - timedelta(hours=1)

    # 上报上一个小时的数据
    response = await send_telemetry_report(prev_hour, current_hour)
    if response.success:
        await finish_with(matcher, message=f"遥测数据上报成功: {prev_hour} - {current_hour}")
    else:
        await finish_with(matcher, message=f"遥测数据上报失败: {response.message}")


@on_command("model_test", aliases={"model-test"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    model_names = cmd_content.strip().split()
    if "-g" in model_names:
        model_names.remove("-g")
        use_group_name = True
    else:
        use_group_name = False

    if "--stream" in model_names:
        stream_mode = True
        model_names.remove("--stream")
    else:
        stream_mode = False

    if "--use-system" in model_names:
        use_system = True
        model_names.remove("--use-system")
    else:
        use_system = False

    if not model_names:
        await finish_with(matcher, message="请指定要测试的模型名 (model_test <model_name1> <model_name2> ...)")

    test_model_groups: List[ModelConfigGroup] = []
    if use_group_name:
        # 处理模型组名称的通配符匹配
        for group_name in model_names:
            if "*" in group_name:
                pattern = group_name.replace("*", ".*")

                matching_groups = [g for g in config.MODEL_GROUPS if re.match(pattern, g)]
                test_model_groups.extend(
                    [config.MODEL_GROUPS[g] for g in matching_groups if config.MODEL_GROUPS[g].MODEL_TYPE == "chat"],
                )
            elif group_name in config.MODEL_GROUPS:
                if config.MODEL_GROUPS[group_name].MODEL_TYPE == "chat":
                    test_model_groups.append(config.MODEL_GROUPS[group_name])
    else:
        # 处理模型名称的通配符匹配
        for model_name in model_names:
            if "*" in model_name:
                pattern = model_name.replace("*", ".*")

                matching_groups = [
                    g for g in config.MODEL_GROUPS.values() if g.MODEL_TYPE == "chat" and re.match(pattern, g.CHAT_MODEL)
                ]
                test_model_groups.extend(matching_groups)
            else:
                matching_groups = [
                    g for g in config.MODEL_GROUPS.values() if model_name == g.CHAT_MODEL and g.MODEL_TYPE == "chat"
                ]
                test_model_groups.extend(matching_groups)

    if not test_model_groups:
        await finish_with(matcher, message="未找到符合条件的模型组")

    await matcher.send(f"正在准备测试 {len(test_model_groups)} 个模型组...")

    model_test_success_result_map: Dict[str, int] = {}
    model_test_fail_result_map: Dict[str, int] = {}
    model_speed_map: Dict[str, List[float]] = {}

    for model_group in test_model_groups:
        if model_group.CHAT_MODEL not in model_test_success_result_map:
            model_test_success_result_map[model_group.CHAT_MODEL] = 0
        if model_group.CHAT_MODEL not in model_test_fail_result_map:
            model_test_fail_result_map[model_group.CHAT_MODEL] = 0
        if model_group.CHAT_MODEL not in model_speed_map:
            model_speed_map[model_group.CHAT_MODEL] = []

        try:
            start_time = time.time()
            messages = [{"role": "user", "content": "Repeat the following text without any thinking or explanation: Test"}]
            if use_system:
                messages.insert(
                    0,
                    {"role": "system", "content": "You are a helpful assistant that follows instructions precisely."},
                )
            llm_response: OpenAIResponse = await gen_openai_chat_response(
                model=model_group.CHAT_MODEL,
                messages=messages,
                base_url=model_group.BASE_URL,
                api_key=model_group.API_KEY,
                stream_mode=stream_mode,
                proxy_url=model_group.CHAT_PROXY,
            )
            end_time = time.time()
            assert llm_response.response_content
            model_test_success_result_map[model_group.CHAT_MODEL] += 1
            model_speed_map[model_group.CHAT_MODEL].append(end_time - start_time)
        except Exception as e:
            logger.error(f"测试 {model_group.CHAT_MODEL} 失败: {e}")
            model_test_fail_result_map[model_group.CHAT_MODEL] += 1

    # 构建测试结果输出
    result_lines = ["=> [模型测试结果]"]
    for model_name in set(list(model_test_success_result_map.keys()) + list(model_test_fail_result_map.keys())):
        success = model_test_success_result_map.get(model_name, 0)
        fail = model_test_fail_result_map.get(model_name, 0)
        status = "✅ 通过" if success > 0 and fail == 0 else "❌ 失败" if fail > 0 else "⚠️ 未知"

        # 添加速度信息
        speed_info = ""
        if model_speed_map.get(model_name):
            speeds = model_speed_map[model_name]
            avg_speed = sum(speeds) / len(speeds)
            if len(speeds) > 1:
                min_speed = min(speeds)
                max_speed = max(speeds)
                speed_info = f" | 速度: {avg_speed:.2f}s (最快: {min_speed:.2f}s, 最慢: {max_speed:.2f}s)"
            else:
                speed_info = f" | 速度: {avg_speed:.2f}s"

        result_lines.append(f"{status} {model_name}: (成功: {success}, 失败: {fail}){speed_info}")

    await finish_with(matcher, message="\n".join(result_lines))


@on_command("clear_sandbox_cache", aliases={"clear-sandbox-cache", "na_csc", "na-csc"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    """清理沙盒环境的缓存目录"""
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    cleared_size = 0
    cleared_files = 0

    # 清理 pip 缓存目录
    pip_cache_path = Path(SANDBOX_PIP_CACHE_DIR)
    if pip_cache_path.exists():
        for root, _, files in os.walk(SANDBOX_PIP_CACHE_DIR):
            root_path = Path(root)
            for file in files:
                file_path = root_path / file
                try:
                    file_size = file_path.stat().st_size
                    file_path.unlink()
                    cleared_size += file_size
                    cleared_files += 1
                except Exception as e:
                    logger.error(f"删除文件失败 {file_path}: {e}")

        # 清空目录但保留目录本身
        for item in pip_cache_path.iterdir():
            if item.is_dir():
                try:
                    shutil.rmtree(item)
                except Exception as e:
                    logger.error(f"删除目录失败 {item}: {e}")

    # 清理包目录
    package_path = Path(SANDBOX_PACKAGE_DIR)
    if package_path.exists():
        for root, _, files in os.walk(SANDBOX_PACKAGE_DIR):
            root_path = Path(root)
            for file in files:
                file_path = root_path / file
                try:
                    file_size = file_path.stat().st_size
                    file_path.unlink()
                    cleared_size += file_size
                    cleared_files += 1
                except Exception as e:
                    logger.error(f"删除文件失败 {file_path}: {e}")

        # 清空目录但保留目录本身
        for item in package_path.iterdir():
            if item.is_dir():
                try:
                    shutil.rmtree(item)
                except Exception as e:
                    logger.error(f"删除目录失败 {item}: {e}")

    # 创建目录（如果不存在）
    Path(SANDBOX_PIP_CACHE_DIR).mkdir(parents=True, exist_ok=True)
    Path(SANDBOX_PACKAGE_DIR).mkdir(parents=True, exist_ok=True)

    size_in_mb = cleared_size / (1024 * 1024)
    await finish_with(
        matcher,
        message=f"沙盒缓存清理完成！\n已清理文件：{cleared_files} 个\n释放空间：{size_in_mb:.2f} MB",
    )


# ! 高风险命令
@on_command("docker_restart", aliases={"docker-restart"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if not OsEnv.RUN_IN_DOCKER:
        await finish_with(matcher, message="当前环境不在 Docker 容器中，无法执行此操作")

    container_name: str = cmd_content or "nekro_agent"
    os.system(f"docker restart {container_name}")


@on_command("docker_logs", aliases={"docker-logs"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if not OsEnv.RUN_IN_DOCKER:
        await finish_with(matcher, message="当前环境不在 Docker 容器中，无法执行此操作")

    lines_limit: int = 100
    container_name: str = cmd_content or "nekro_agent"
    logs = os.popen(f"docker logs {container_name} --tail {lines_limit}").read()
    await finish_with(matcher, message=f"容器日志: \n{logs}")


@on_command("sh", priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    outputs = os.popen(cmd_content).read()
    await finish_with(matcher, message=f"命令 `{cmd_content}` 输出: \n{outputs or '<Empty>'}")


DB_RESET_LATEST_TRIGGER_TIME: float = 0


@on_command("nekro_db_reset", aliases={"nekro-db-reset"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    global DB_RESET_LATEST_TRIGGER_TIME

    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)
    args = cmd_content.split(" ")

    if time.time() - DB_RESET_LATEST_TRIGGER_TIME > 60:
        DB_RESET_LATEST_TRIGGER_TIME = time.time()
        await finish_with(
            matcher,
            message="正在准备执行数据库重置操作！确认继续重置请在 1 分钟内再次使用本命令并使用 `-y` 参数确认",
        )
        return

    if "-y" in args:
        args.remove("-y")
        if len(args) > 1:
            await finish_with(matcher, message="参数不合法")
        if len(args) == 1:
            await reset_db(args[0])
            await finish_with(matcher, message=f"数据表 `{args[0]}` 重置完成")
        else:
            await reset_db()
            await finish_with(matcher, message="数据库重置完成")
    else:
        await finish_with(matcher, message="请使用 `-y` 参数确认重置数据库")


@on_command("github_stars_check", aliases={"github-stars-check"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    """检查用户是否已Star官方GitHub仓库"""
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    try:
        result = await check_official_repos_starred()

        if not result.success:
            await finish_with(matcher, message=f"检查GitHub Star状态失败: {result.message}")

        if not result.data:
            await finish_with(matcher, message="获取Star状态数据为空")

        # 构建响应消息
        starred = ", ".join(result.data.starred_repositories) if result.data.starred_repositories else "无"
        unstarred = ", ".join(result.data.unstarred_repositories) if result.data.unstarred_repositories else "无"
        status = "已Star所有官方仓库" if result.data.all_starred else "还有未Star的官方仓库"

        message = f"=> [GitHub Star 状态]\n状态: {status}\n已Star: {starred}\n未Star: {unstarred}"

    except Exception as e:
        logger.error(f"检查GitHub Star状态时发生错误: {e}")
        await finish_with(matcher, message=f"执行失败: {e}")
    else:
        await finish_with(matcher, message=message)


@on_command("log_err_list", aliases={"log-err-list", "log_err_ls", "log-err-ls"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    """查看错误日志列表"""
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    from nekro_agent.services.agent.run_agent import RECENT_ERR_LOGS

    # 解析参数，支持分页
    args = cmd_content.strip().split()
    page = 1
    page_size = 10
    use_dir_files = False

    for i, _arg in enumerate(args):
        if _arg == "-p" and i + 1 < len(args):
            try:
                page = int(args[i + 1])
            except ValueError:
                await finish_with(matcher, message="⚠️ 分页参数格式错误")
        elif _arg == "-s" and i + 1 < len(args):
            try:
                page_size = int(args[i + 1])
            except ValueError:
                await finish_with(matcher, message="⚠️ 每页显示数量参数格式错误")
        elif _arg == "-a" or _arg == "--all":
            use_dir_files = True

    # 确保页码和页大小合法
    page = max(1, page)
    page_size = max(1, min(50, page_size))

    # 获取所有错误日志
    if use_dir_files:
        # 从错误日志目录读取所有文件
        log_dir = Path(PROMPT_ERROR_LOG_DIR)
        if not log_dir.exists():
            await finish_with(matcher, message="⚠️ 错误日志目录不存在")

        all_logs = []
        for file_path in log_dir.glob("*.json"):
            all_logs.append(file_path)

        # 按修改时间从新到旧排序
        logs = sorted(all_logs, key=lambda p: p.stat().st_mtime, reverse=True)
    else:
        # 使用缓存的最近日志列表，从新到旧排序
        logs = list(RECENT_ERR_LOGS)

    total_logs = len(logs)
    total_pages = (total_logs + page_size - 1) // page_size if total_logs > 0 else 1

    # 确保页码不超过总页数
    page = min(page, total_pages)

    # 计算当前页的日志
    start_idx = (page - 1) * page_size
    end_idx = min(start_idx + page_size, total_logs)
    current_page_logs = logs[start_idx:end_idx]

    # 构建响应消息
    if not current_page_logs:
        await finish_with(matcher, message="📭 没有错误日志记录")

    result_lines = [f"📋 错误日志列表 (第 {page}/{total_pages} 页，共 {total_logs} 条):"]

    for i, log_path in enumerate(current_page_logs, start=start_idx + 1):
        # 获取文件修改时间
        try:
            mod_time = datetime.fromtimestamp(log_path.stat().st_mtime).strftime("%m-%d %H:%M:%S")
            result_lines.append(f"{i}. 📄 [{mod_time}] {log_path.name}")
        except:
            result_lines.append(f"{i}. 📄 {log_path.name}")

    result_lines.append("\n🔍 使用方法:")
    result_lines.append("log_err_list -p <页码> -s <每页数量>: 查看最近错误日志列表")
    result_lines.append("log_err_list -a/--all: 查看错误日志目录中的所有日志文件")
    result_lines.append("log_chat_test <日志索引/文件名> [-g <模型组名>] [--stream]: 使用错误日志内容测试请求")

    await finish_with(matcher, message="\n".join(result_lines))


@on_command("log_chat_test", aliases={"log-chat-test"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    """使用错误日志中的对话测试LLM请求"""
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    from nekro_agent.services.agent.run_agent import RECENT_ERR_LOGS

    # 解析参数
    args = cmd_content.strip().split()
    if not args:
        await finish_with(matcher, message="⚠️ 请指定要测试的日志索引或文件名")

    log_identifier = args[0]
    model_group_name = config.USE_MODEL_GROUP  # 默认使用主模型组
    use_stream_mode = False  # 是否使用流式请求

    # 检查是否指定了模型组或流式模式
    i = 1
    while i < len(args):
        if args[i] == "-g" and i + 1 < len(args):
            model_group_name = args[i + 1]
            i += 2
        elif args[i] == "--stream" or args[i] == "-s":
            use_stream_mode = True
            i += 1
        else:
            i += 1

    # 验证模型组是否存在
    if model_group_name not in config.MODEL_GROUPS:
        await finish_with(matcher, message=f"⚠️ 指定的模型组 '{model_group_name}' 不存在")

    model_group = config.MODEL_GROUPS[model_group_name]

    # 查找目标日志文件
    log_path = None
    try:
        # 尝试作为索引解析
        idx = int(log_identifier) - 1
        logs = list(RECENT_ERR_LOGS)
        if 0 <= idx < len(logs):
            log_path = logs[idx]
    except ValueError:
        # 尝试作为文件名匹配队列中的文件
        for p in RECENT_ERR_LOGS:
            if log_identifier == p.name:
                log_path = p
                break

        # 如果仍未找到，尝试直接在错误日志目录中查找
        if not log_path:
            direct_path = Path(PROMPT_ERROR_LOG_DIR) / log_identifier
            if direct_path.exists() and direct_path.is_file():
                log_path = direct_path

    if not log_path and not log_identifier.endswith(".json"):
        direct_path = Path(PROMPT_ERROR_LOG_DIR) / f"{log_identifier}.json"
        if direct_path.exists() and direct_path.is_file():
            log_path = direct_path

    if not log_path:
        await finish_with(
            matcher,
            message=f"⚠️ 未找到指定的日志: {log_identifier}\n提示: 可以使用log_err_list命令查看最近的错误日志，或直接指定错误日志目录中的文件名",
        )

    # 检查日志文件是否存在
    if not log_path.exists():
        await finish_with(matcher, message=f"⚠️ 日志文件不存在: {log_path.name}")

    # 读取日志文件内容
    try:
        log_content = log_path.read_text(encoding="utf-8")
        log_data = json.loads(log_content)
    except Exception as e:
        await finish_with(matcher, message=f"⚠️ 解析日志文件失败: {e}")

    # 从日志中提取messages
    try:
        messages = log_data["request"]["messages"]
    except KeyError:
        # 尝试旧格式
        try:
            messages = log_data.get("messages", [])
            if not messages:
                await finish_with(matcher, message=f"⚠️ 日志中未找到有效的对话内容: {log_path.name}")
        except (KeyError, AttributeError):
            await finish_with(matcher, message=f"⚠️ 日志格式不合法或未找到有效的对话内容: {log_path.name}")

    # 测试前的提示信息
    stream_info = "（流式模式）" if use_stream_mode else ""
    testing_message = (
        f"🚀 正在使用 {model_group.CHAT_MODEL} ({model_group_name}) {stream_info}\n测试请求日志: {log_path.name}..."
    )
    await matcher.send(testing_message)

    # 发起测试请求
    start_time = time.time()
    try:
        llm_response: OpenAIResponse = await gen_openai_chat_response(
            model=model_group.CHAT_MODEL,
            messages=messages,
            base_url=model_group.BASE_URL,
            api_key=model_group.API_KEY,
            stream_mode=use_stream_mode,
            proxy_url=model_group.CHAT_PROXY,
            max_wait_time=config.AI_GENERATE_TIMEOUT,
        )
        end_time = time.time()

        # 获取响应总长度
        total_length = len(llm_response.response_content)

        # 截取响应结果的前 64 个字符
        preview = (
            llm_response.response_content[:64] + "..."
            if len(llm_response.response_content) > 64
            else llm_response.response_content
        )

        # 构建响应消息
        elapsed = end_time - start_time
        # 根据响应时间添加不同的emoji
        speed_emoji = "⚡" if elapsed < 1 else "🚀" if elapsed < 3 else "🏃" if elapsed < 5 else "🐢"

        # 根据长度选择emoji
        length_emoji = "📏" if total_length < 100 else "📊" if total_length < 500 else "📜" if total_length < 2000 else "📚"

        result_message = (
            f"🟢 测试成功！\n"
            f"📊 测试结果:\n"
            f"- 模型: {model_group.CHAT_MODEL}\n"
            f"- 流式模式: {use_stream_mode}\n"
            f"- 耗时: {speed_emoji} {elapsed:.2f}s\n"
            f"- 响应预览>\n{preview}\n======\n"
            f"响应总长度: {length_emoji} {total_length} 字符\n"
        )

    except Exception as e:
        end_time = time.time()
        elapsed = end_time - start_time

        safe_error: str = str(e).replace(model_group.API_KEY, "[API_KEY]").replace(model_group.BASE_URL, "[BASE_URL]")

        await finish_with(
            matcher,
            message=(
                f"🔴 测试失败！\n"
                f"📊 测试结果:\n"
                f"- 模型: {model_group.CHAT_MODEL}\n"
                f"- 流式模式: {use_stream_mode}\n"
                f"- 耗时: {elapsed:.2f}s\n"
                f"- 错误信息: {safe_error!s}"
            ),
        )
    else:
        await finish_with(matcher, message=result_message)



from typing import Any, Dict, Optional

from pydantic import BaseModel


class WebhookRequest(BaseModel):
    """Webhook 请求"""

    headers: Dict[str, str]
    body: Dict[str, Any]


class AgentCtx(BaseModel):
    container_key: Optional[str] = None
    from_chat_key: str
    extra_data: Dict[str, Any] = {}
    webhook_request: Optional[WebhookRequest] = None



import re
from typing import Tuple

from pydantic import BaseModel


class ParsedCodeRunData(BaseModel):
    raw_content: str
    code_content: str
    thought_chain: str


def parse_chat_response(raw_content: str) -> ParsedCodeRunData:
    """
    解析聊天响应内容，提取思维链和代码内容。

    Args:
        raw_content: 原始响应内容

    Returns:
        ParsedCodeRunData对象, 即代码执行数据
    """
    # 清理和准备内容
    cleaned_content = raw_content.strip()
    thought_chain = ""
    code_content = ""

    # 检查是否同时存在<think>和</think>标签
    has_think_tags = "<think>" in cleaned_content and "</think>" in cleaned_content

    if has_think_tags:
        # 如果存在思维链，先提取思维链
        think_pattern = re.compile(r"<think>(.*?)</think>", re.DOTALL)
        think_match = think_pattern.search(cleaned_content)

        if think_match:
            # 提取思维链内容并清理可能存在的嵌套标签
            thought_chain = think_match.group(1).strip()
            thought_chain = re.sub(r"</?think>", "", thought_chain)

            # 在思维链之后查找代码块
            remaining_content = cleaned_content[think_match.end() :].strip()
            code_pattern = re.compile(r"```(?:python)?\s*(.*?)```(?:\s*$|(?=\s*```\s*$))", re.DOTALL)
            code_match = code_pattern.search(remaining_content)

            if code_match:
                code_content = code_match.group(1).strip()

    # 如果通过思维链匹配没有提取到代码内容，尝试直接匹配代码块
    if not code_content:
        # 提取最长的完整代码块（从```python到最后一个```）
        code_pattern = re.compile(r"```(?:python)?\s*(.*?)```(?:\s*$|(?=\s*```\s*$))", re.DOTALL)
        code_matches = code_pattern.finditer(cleaned_content)

        # 获取所有匹配结果
        matches_list = list(code_matches)
        if matches_list:
            # 选择最长的代码块
            code_content = max((match.group(1).strip() for match in matches_list), key=len)
        else:
            # 如果没有代码块，移除思维链部分后的内容作为代码
            content_without_think = re.sub(r"<think>.*?</think>", "", cleaned_content, flags=re.DOTALL).strip()
            code_content = content_without_think

    # 如果依然没有代码内容，则搜索全部响应内容取最长的代码块
    if not code_content:
        code_pattern = re.compile(r"```(?:python)?\s*(.*?)```(?:\s*$|(?=\s*```\s*$))", re.DOTALL)
        code_matches = code_pattern.finditer(cleaned_content)
        matches_list = list(code_matches)
        if matches_list:
            code_content = max((match.group(1).strip() for match in matches_list), key=len)

    if code_content.strip().startswith("```python"):
        code_content = code_content.strip()[len("```python") :].strip()
    if code_content.strip().endswith("```"):
        code_content = code_content.strip()[: -len("```")].strip()

    return ParsedCodeRunData(raw_content=raw_content, code_content=fix_code_content(code_content), thought_chain=thought_chain)


def fix_code_content(code_content: str) -> str:
    """修复代码内容"""
    # 修正代码块去掉所有 from plugins ... import ... 开头的行
    code_content = re.sub(r"^from plugins.*\n", "", code_content, flags=re.MULTILINE)
    code_content = re.sub(r"^from predefined_methods .*\n", "", code_content, flags=re.MULTILINE)
    code_content = re.sub(r"^from predefined_plugins .*\n", "", code_content, flags=re.MULTILINE)
    code_content = re.sub(r"^from core\.api.*\n", "", code_content, flags=re.MULTILINE)
    # 修正代码块去掉所有 from plugin_manager ... import ... 开头的行
    return re.sub(r"^from plugin_manager.*\n", "", code_content, flags=re.MULTILINE)


def fix_raw_response(raw_response: str) -> str:
    """修复原始响应"""
    # logger.debug(f"Raw response: {raw_response}")
    # 修正基本 at 格式
    raw_response = raw_response.replace("[qq:", "[@qq:")
    raw_response = raw_response.replace("@[qq:", "[@qq:")
    # 修正 [@qq:123456] -> [@qq:123456@]
    raw_response = re.sub(r"\[@qq:(\d+)\]", r"[@qq:\1@]", raw_response)
    # 修正 [@qq:123456;nickname:Abc] -> [@qq:123456@]
    raw_response = re.sub(r"\[@qq:(\d+);nickname[\=\:](.+)\]", r"[@qq:\1@]", raw_response)
    # 修正 [@123456] -> [@qq:123456@]
    raw_response = re.sub(r"\[@(\d+)\]", r"[@qq:\1@]", raw_response)
    # 修正 (@qq:123456@) -> [@qq:123456@]
    raw_response = re.sub(r"\(@qq:(\d+)@\)", r"[@qq:\1@]", raw_response)
    # 修正 (@qq:123456) -> [@qq:123456@]
    raw_response = re.sub(r"\(@qq:(\d+)\)", r"[@qq:\1@]", raw_response)
    # 修正 (@123456@) -> [@qq:123456@]
    raw_response = re.sub(r"\( ?@(\d+)@ ?\)", r"[@qq:\1@]", raw_response)
    # 修正  <7e56b348 | At:[@qq:xxx@]> -> [@qq:xxx@]
    raw_response = re.sub(r"<\w{8} ?\| At:\[@qq:(\d+)@\]>", r"[@qq:\1@]", raw_response)
    # 修正 (@[@qq:123456@]) -> [@qq:123456@]
    raw_response = re.sub(r"\(@\[@qq:(\d+)@\]\)", r"[@qq:\1@]", raw_response)
    # 修正 <@123456> -> [@qq:123456@]
    raw_response = re.sub(r"<@(\d+)>", r"[@qq:\1@]", raw_response)
    # 修正 @123456@) -> [@qq:123456@]
    raw_response = re.sub(r"@(\d+)@\)", r"[@qq:\1@]", raw_response)
    # 修正 (@123456) -> [@qq:123456@]
    raw_response = re.sub(r"\( ?@(\d+) ?\)", r"[@qq:\1@]", raw_response)
    # 修正 @123456@) -> [@qq:123456@]
    raw_response = re.sub(r"@(\d+)@ ?\)", r"[@qq:\1@]", raw_response)

    # 处理类似 `<1952b262 | message separator>` 模型幻觉续写的情况，截断其后的所有内容
    reg = r"<\w{8} \| message separator>"
    match = re.search(reg, raw_response)
    if match:
        raw_response = raw_response[: match.start()]

    # 检查是否存在多个 <think> 标签
    tags = ["<think>", "</think>"]
    for tag in tags:
        if raw_response.count(tag) > 1 and raw_response.endswith(tag):
            raw_response = raw_response[: -len(tag)]

    # logger.debug(f"Fixed raw response: {raw_response}")
    return raw_response.strip()



from typing import Callable

from .base import SandboxMethodType


def get_sandbox_method_type(method: Callable) -> SandboxMethodType:
    if hasattr(method, "_method_type"):
        try:
            return SandboxMethodType(method._method_type)  # noqa: SLF001
        except ValueError as e:
            raise ValueError(f"方法 {method.__name__} 的 _method_type 属性值无效。") from e
    raise AttributeError(f"方法 {method.__name__} 没有 _method_type 属性。")



"""Schema 类型定义

此模块提供了 Nekro-Agent 的类型定义。
"""

from typing import List, Union

from nekro_agent.schemas.agent_ctx import AgentCtx, WebhookRequest
from nekro_agent.schemas.agent_message import (
    AgentMessageSegment,
    AgentMessageSegmentType,
)

__all__ = [
    "AgentCtx",
    "AgentMessageSegment",
    "AgentMessageSegmentType",
    "MessageContent",
    "WebhookRequest",
]

# 消息内容类型
MessageContent = Union[str, List[AgentMessageSegment]]



"""沙盒环境下的扩展方法调用代理"""

import importlib
import os
import pickle as _pickle
import subprocess
import sys
import urllib.parse
from importlib.metadata import PackageNotFoundError, distributions
from pathlib import Path
from typing import Any, Callable, Dict, Optional, Tuple

import matplotlib.pyplot as plt
import requests as _requests
from packaging.specifiers import SpecifierSet
from packaging.version import parse

# 设置中文字体
plt.rcParams["font.sans-serif"] = ["SimHei", "DejaVu Sans", "Arial Unicode MS", "sans-serif"]
plt.rcParams["axes.unicode_minus"] = False

CHAT_API = "{CHAT_API}"
CONTAINER_KEY = "{CONTAINER_KEY}"
FROM_CHAT_KEY = "{FROM_CHAT_KEY}"
RPC_SECRET_KEY = "{RPC_SECRET_KEY}"


def __extension_method_proxy(method: Callable):
    """扩展方法代理执行器"""

    def acutely_call_method(*args: Tuple[Any], **kwargs: Dict[str, Any]):
        """Agent 执行沙盒扩展方法时实际调用的方法"""

        body = {"method": method.__name__, "args": args, "kwargs": kwargs}
        data: bytes = _pickle.dumps(body)
        response = _requests.post(
            f"{CHAT_API}/ext/rpc_exec?container_key={CONTAINER_KEY}&from_chat_key={FROM_CHAT_KEY}",
            data=data,
            headers={
                "Content-Type": "application/octet-stream",
                "X-RPC-Token": RPC_SECRET_KEY,
            },
        )
        if response.status_code == 200:
            if response.headers.get("Run-Error") and response.headers["Run-Error"].lower() == "true":
                print(
                    f"The method `{method.__name__}` returned an error:\n{response.text}",
                )
                exit(1)
            ret_data = _pickle.loads(response.content)
            if response.headers.get("Method-Type") == "agent":
                print(
                    f"The agent method `{method.__name__}` returned:\n{ret_data}\n[result end]\nPlease continue to generate an appropriate response based on the above information.",
                )
                exit(8)
            if response.headers.get("Method-Type") == "multimodal_agent":
                print(
                    f"The multimodal agent method `{method.__name__}` returned:\n{ret_data}\n[result end]",
                )
                exit(11)
            return ret_data
        raise Exception(f"Plugin RPC method `{method.__name__}` call failed: {response.status_code}")

    return acutely_call_method


def dynamic_importer(
    package_spec: str,
    import_name: Optional[str] = None,
    mirror: Optional[str] = "https://pypi.tuna.tsinghua.edu.cn/simple",
    trusted_host: bool = True,
    timeout: int = 300,
    repo_dir: Optional[str] = "/app/packages",
) -> Any:
    """动态安装并导入Python包

    Args:
        package_spec: 包名称和版本规范 (如 "requests" 或 "numpy==1.21.0")
        import_name: 导入名称（如果与包名不同）
        mirror: PyPI镜像源URL
        trusted_host: 是否信任镜像源主机
        timeout: 安装超时时间（秒）
        repo_dir: 持久化存储目录 (默认使用系统路径)

    Returns:
        导入的模块对象

    Raises:
        RuntimeError: 安装失败时抛出
        ImportError: 导入失败时抛出
    """
    # 提取基础包名和版本约束
    package_name = package_spec.strip()
    version_spec = ""
    for sep in ["==", ">=", "<=", "!=", ">", "<", "~="]:
        if sep in package_name:
            package_name, version_spec = package_name.split(sep, 1)
            package_name = package_name.strip()
            version_spec = f"{sep}{version_spec.strip()}"
            break

    # 配置持久化仓库路径
    if repo_dir:
        repo_path = Path(repo_dir)
        repo_path.mkdir(parents=True, exist_ok=True)
        repo_site_packages = repo_path / "site-packages"
        repo_site_packages.mkdir(parents=True, exist_ok=True)

        # 添加路径到sys.path
        for path in [str(repo_path), str(repo_site_packages)]:
            if path not in sys.path:
                sys.path.insert(0, path)

    # 检查是否已安装符合条件的版本
    need_install = True
    try:
        # 处理distributions函数的path参数
        if repo_dir:
            path_list = [repo_dir]
            dists = distributions(path=path_list)
        else:
            dists = distributions()

        for dist in dists:
            metadata = dist.metadata
            dist_name = getattr(metadata, "Name", None) or dist.name
            if dist_name.lower() == package_name.lower() and (
                not version_spec or parse(dist.version) in SpecifierSet(version_spec)
            ):
                need_install = False
                break
    except PackageNotFoundError:
        pass

    # 构建安装命令
    if need_install:
        install_cmd = [
            sys.executable,
            "-m",
            "pip",
            "install",
            "--disable-pip-version-check",
            "--quiet",
            "--no-input",
            "--no-warn-script-location",
        ]

        # 添加持久化目录参数
        if repo_dir:
            install_cmd += ["--target", repo_dir]

        # 添加镜像源配置
        if mirror:
            install_cmd += ["--index-url", mirror]
            if trusted_host:
                host = urllib.parse.urlparse(mirror).hostname
                if host:
                    install_cmd += ["--trusted-host", host]

        install_cmd.append(package_spec)

        # 执行安装
        try:
            subprocess.run(install_cmd, capture_output=True, text=True, check=True, timeout=timeout)
        except subprocess.CalledProcessError as e:
            error_msg = _parse_pip_error(e.stderr or e.stdout)
            print(f"安装 {package_spec} 失败: {error_msg}")
            exit(1)
        except subprocess.TimeoutExpired:
            print(f"安装 {package_spec} 超时（{timeout}秒），请检查网络连接")
            exit(1)

    # 确定导入模块名称
    module_name = import_name if import_name is not None else package_name

    # 动态导入模块
    try:
        module = importlib.import_module(module_name)
        module = importlib.reload(module)  # 确保加载最新版本
    except ImportError:
        # 尝试刷新导入路径
        if repo_dir:
            import site

            site.addsitedir(repo_dir)
            try:
                module = importlib.import_module(module_name)
            except ImportError:
                print(f"安装成功但无法导入 {module_name}，请确认模块名正确")
                exit(1)
        else:
            print(f"安装成功但无法导入 {module_name}，请确认模块名正确")
            exit(1)

    return module


def _parse_pip_error(output: str) -> str:
    """解析pip错误信息"""
    patterns = {
        "No matching distribution": "包不存在或版本不可用",
        "Could not find a version": "找不到指定版本",
        "SSL: CERTIFICATE_VERIFY_FAILED": "SSL验证失败，尝试使用 trusted_host=True",
        "403 Client Error": "访问被拒绝（可能镜像源需要认证）",
        "404 Client Error": "资源不存在",
        "Connection refused": "连接被拒绝，检查镜像源地址",
        "Network is unreachable": "网络不可达",
    }
    for pattern, message in patterns.items():
        if pattern in output:
            return message
    return f"未知错误：{output[:200]}..." if len(output) > 200 else output



import asyncio
import time
from pathlib import Path
from typing import AsyncGenerator, Optional

import aiodocker
from fastapi import APIRouter, Depends, HTTPException
from sse_starlette.sse import EventSourceResponse

from nekro_agent.core.config import config
from nekro_agent.core.logger import logger
from nekro_agent.core.os_env import (
    NAPCAT_ONEBOT_ADAPTER_DIR,
    ONEBOT_ACCESS_TOKEN,
    OsEnv,
)
from nekro_agent.models.db_user import DBUser
from nekro_agent.schemas.message import Ret
from nekro_agent.services.user.deps import get_current_active_user
from nekro_agent.services.user.perm import Role, require_role

router = APIRouter(prefix="/napcat", tags=["NapCat"])


@router.get("/onebot-token")
@require_role(Role.Admin)
async def get_onebot_token(_current_user: DBUser = Depends(get_current_active_user)) -> Ret:
    """获取 OneBot 访问令牌"""
    if not ONEBOT_ACCESS_TOKEN:
        return Ret.success(data=None, msg="未设置 OneBot 访问令牌")
    return Ret.success(data=ONEBOT_ACCESS_TOKEN, msg="获取 OneBot 访问令牌成功")


async def get_docker():
    """获取或创建 Docker 客户端"""
    return aiodocker.Docker()


async def get_container():
    """获取 NapCat 容器实例"""
    if not config.NAPCAT_CONTAINER_NAME:
        raise HTTPException(status_code=500, detail="未设置 NapCat 容器名称")
    try:
        client = await get_docker()
        return await client.containers.get(config.NAPCAT_CONTAINER_NAME)
    except Exception as e:
        logger.error(f"获取容器失败: {e!s}")
        raise HTTPException(status_code=500, detail=str(e)) from e


@router.get("/status")
@require_role(Role.Admin)
async def get_status(_current_user: DBUser = Depends(get_current_active_user)) -> Ret:
    """获取容器状态"""
    try:
        container = await get_container()
        state = (await container.show())["State"]
        return Ret.success(
            data={
                "running": state["Running"],
                "started_at": state["StartedAt"],
            },
            msg="获取容器状态成功",
        )
    except HTTPException as e:
        return Ret.fail(e.detail)
    except Exception as e:
        logger.error(f"获取状态失败: {e!s}")
        return Ret.fail(str(e))


@router.get("/logs")
@require_role(Role.Admin)
async def get_logs(tail: Optional[int] = 100, _current_user: DBUser = Depends(get_current_active_user)) -> Ret:
    """获取最近的容器日志"""
    try:
        container = await get_container()
        logs = await container.log(stdout=True, stderr=True, tail=tail)
        return Ret.success(data=logs, msg="获取日志成功")
    except HTTPException as e:
        return Ret.fail(e.detail)
    except Exception as e:
        logger.error(f"获取日志失败: {e!s}")
        return Ret.fail(str(e))


@router.get("/logs/stream")
@require_role(Role.Admin)
async def stream_logs(_current_user: DBUser = Depends(get_current_active_user)) -> EventSourceResponse:
    """实时日志流"""
    try:

        async def generate() -> AsyncGenerator[str, None]:
            try:
                container = await get_container()
                # 先发送最近的日志
                initial_logs = await container.log(stdout=True, stderr=True, tail=100, timestamps=False)
                init_time = time.time()
                for log in initial_logs:
                    # if log.startswith("20"):  # 移除时间戳前缀
                    #     log = log.split("Z ", 1)[1]
                    yield log
                    await asyncio.sleep(0.01)  # 避免发送过快

                # 然后开始实时流式传输
                async for log in container.log(stdout=True, stderr=True, follow=True, since=int(init_time), timestamps=False):
                    # if log.startswith("20"):
                    #     log = log.split("Z ", 1)[1]
                    yield log
                    await asyncio.sleep(0.01)  # 避免发送过快
            except Exception as e:
                logger.error(f"日志流异常: {e!s}")
                raise

        return EventSourceResponse(generate())
    except Exception as e:
        logger.error(f"日志流异常: {e!s}")
        raise HTTPException(status_code=500, detail=str(e)) from e


ADAPTER_TEMPLATE = """
{{
  "network": {{
    "httpServers": [],
    "httpSseServers": [],
    "httpClients": [],
    "websocketServers": [],
    "websocketClients": [
      {{
        "enable": true,
        "name": "ws",
        "url": "ws://{instance_name}:{port}/onebot/v11/ws",
        "reportSelfMessage": false,
        "messagePostFormat": "array",
        "token": "{token}",
        "debug": false,
        "heartInterval": 30000,
        "reconnectInterval": 3000
      }}
    ],
    "plugins": []
  }},
  "musicSignUrl": "",
  "enableLocalFile2Url": false,
  "parseMultMsg": false
}}
""".strip()


@router.post("/restart")
@require_role(Role.Admin)
async def restart(_current_user: DBUser = Depends(get_current_active_user)) -> Ret:
    """重启容器"""
    if config.BOT_QQ:
        adapter_file = Path(NAPCAT_ONEBOT_ADAPTER_DIR) / f"onebot11_{config.BOT_QQ}.json"
        if not adapter_file.exists():
            logger.info(f"未找到适配器文件: {adapter_file}, 自动创建...")
            adapter_file.write_text(
                ADAPTER_TEMPLATE.format(
                    token=ONEBOT_ACCESS_TOKEN,
                    instance_name=OsEnv.INSTANCE_NAME,
                    port=OsEnv.EXPOSE_PORT,
                ),
                encoding="utf-8",
            )
    try:
        container = await get_container()
        # 发送重启命令后立即返回，不等待重启完成
        asyncio.create_task(container.restart(timeout=30))
        return Ret.success(data=True, msg="重启命令已发送")
    except HTTPException as e:
        return Ret.fail(e.detail)
    except Exception as e:
        logger.error(f"重启失败: {e!s}")
        return Ret.fail(str(e))



from datetime import datetime
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

from nekro_agent.systems.cloud.schemas.base import BasicResponse


class PluginBase(BaseModel):
    """插件基础信息模型"""

    id: str
    name: str
    moduleName: str
    description: str
    author: str
    hasWebhook: bool
    homepageUrl: Optional[str] = None
    githubUrl: Optional[str] = None
    cloneUrl: Optional[str] = None
    licenseType: Optional[str] = None
    created_at: str = Field(alias="createdAt")
    updated_at: str = Field(alias="updatedAt")
    icon: Optional[str] = None  # 插件图标URL
    isOwner: Optional[bool] = None  # 是否为当前用户上传的插件


class PluginCreate(BaseModel):
    """创建插件请求模型"""

    name: str
    moduleName: str
    description: str
    author: str
    hasWebhook: bool
    homepageUrl: Optional[str] = None
    githubUrl: Optional[str] = None
    cloneUrl: Optional[str] = None
    licenseType: Optional[str] = None
    isSfw: bool = True
    icon: Optional[str] = None  # 插件图标，可以是Base64或URL


class PluginUpdate(BaseModel):
    """更新插件请求模型"""

    name: Optional[str] = None
    description: Optional[str] = None
    author: Optional[str] = None
    hasWebhook: Optional[bool] = None
    homepageUrl: Optional[str] = None
    githubUrl: Optional[str] = None
    cloneUrl: Optional[str] = None
    licenseType: Optional[str] = None
    isSfw: Optional[bool] = None
    icon: Optional[str] = None  # 插件图标，可以是Base64或URL


class PluginCreateResponse(BasicResponse):
    """创建插件响应模型"""

    data: Optional[Dict[str, Any]] = None


class PluginDetailResponse(BasicResponse):
    """获取插件详情响应模型"""

    data: Optional[PluginBase] = None


class PluginListItem(BaseModel):
    """插件列表项模型"""

    id: str
    name: str
    moduleName: str
    description: str
    author: str
    hasWebhook: bool
    homepageUrl: Optional[str] = None
    githubUrl: Optional[str] = None
    cloneUrl: Optional[str] = None
    licenseType: Optional[str] = None
    createdAt: str
    updatedAt: str
    icon: Optional[str] = None  # 插件图标URL
    isOwner: Optional[bool] = None  # 是否为当前用户上传的插件


class PluginListData(BaseModel):
    """插件列表数据模型"""

    items: List[PluginListItem]
    total: int
    page: int
    pageSize: int


class PluginListResponse(BasicResponse):
    """获取插件列表响应模型"""

    data: Optional[PluginListData] = None


class UserPluginItem(BaseModel):
    """用户插件列表项模型"""

    id: str
    name: str
    moduleName: str


class UserPluginListData(BaseModel):
    """用户插件列表数据模型"""

    items: List[UserPluginItem]
    total: int


class UserPluginListResponse(BasicResponse):
    """获取用户插件列表响应模型"""

    data: Optional[UserPluginListData] = None



"""Nekro Cloud API 客户端模块"""

from .auth import check_official_repos_starred
from .plugin import (
    create_plugin,
    delete_plugin,
    get_plugin,
    list_plugins,
    list_user_plugins,
    update_plugin,
)
from .preset import (
    create_preset,
    delete_preset,
    get_preset,
    list_presets,
    list_user_presets,
    update_preset,
)
from .telemetry import get_community_stats, send_telemetry_report



from typing import Optional, Tuple

from pydantic import BaseModel
from qdrant_client import AsyncQdrantClient
from qdrant_client import models as qdrant_models

from .args import Args
from .config import config
from .logger import logger
from .os_env import OsEnv

_qdrant_client: Optional[AsyncQdrantClient] = None


class QdrantConfig(BaseModel):
    url: str
    api_key: Optional[str]


def get_qdrant_config() -> QdrantConfig:
    """获取 Qdrant 配置"""
    if OsEnv.RUN_IN_DOCKER:
        QDRANT_URL = OsEnv.QDRANT_URL
        QDRANT_API_KEY = OsEnv.QDRANT_API_KEY
    else:
        QDRANT_URL = config.QDRANT_URL
        QDRANT_API_KEY = config.QDRANT_API_KEY
    return QdrantConfig(url=QDRANT_URL, api_key=QDRANT_API_KEY or None)


async def get_qdrant_client() -> AsyncQdrantClient:
    """获取或初始化Qdrant客户端"""
    global _qdrant_client

    if Args.LOAD_TEST:
        logger.warning("在测试模式下，不启用 Qdrant 数据库")
        return None  # type: ignore

    if _qdrant_client is None:
        qdrant_config = get_qdrant_config()
        _qdrant_client = AsyncQdrantClient(
            url=qdrant_config.url,
            api_key=qdrant_config.api_key,
        )
    return _qdrant_client



from nonebot.adapters.onebot.v11 import Bot, Message, MessageEvent
from nonebot.matcher import Matcher
from nonebot.params import CommandArg
from pydantic import Field

from nekro_agent.api import context, core
from nekro_agent.api.plugin import ConfigBase, NekroPlugin, SandboxMethodType
from nekro_agent.api.schemas import AgentCtx
from nekro_agent.matchers.command import command_guard, finish_with, on_command
from nekro_agent.schemas.chat_message import ChatType

plugin = NekroPlugin(
    name="AI 语音插件",
    module_name="ai_voice",
    description="提供AI语音生成功能，支持将文本转为AI合成语音",
    version="0.1.0",
    author="KroMiose",
    url="https://github.com/KroMiose/nekro-agent",
)


@plugin.mount_config()
class AIVoiceConfig(ConfigBase):
    """AI语音配置"""

    AI_VOICE_CHARACTER: str = Field(
        default="lucy-voice-xueling",
        title="AI语音角色",
        description="使用命令 `/ai_voices` 可查看所有可用角色",
    )


# 获取配置
config = plugin.get_config(AIVoiceConfig)


@on_command("ai_voices", aliases={"ai-voices"}, priority=5, block=True).handle()
async def _(matcher: Matcher, event: MessageEvent, bot: Bot, arg: Message = CommandArg()):
    username, cmd_content, chat_key, chat_type = await command_guard(event, bot, arg, matcher)

    if chat_type is ChatType.GROUP:
        tags = await bot.call_api("get_ai_characters", group_id=chat_key.split("_")[1])
        formatted_characters = []
        for tag in tags:
            char_list = []
            for char in tag["characters"]:
                char_list.append(f"ID: {char['character_id']} - {char['character_name']}")
            formatted_characters.append(f"=== {tag['type']} ===\n" + "\n".join(char_list))

        await finish_with(matcher, message="当前可用的 AI 声聊角色: \n\n" + "\n\n".join(formatted_characters))
    else:
        await finish_with(matcher, message="AI 声聊功能仅支持群组")


@plugin.mount_sandbox_method(SandboxMethodType.TOOL, "发送消息语音")
async def ai_voice(_ctx: AgentCtx, chat_key: str, text: str) -> bool:
    """发送消息语音

    Args:
        chat_key (str): 聊天的唯一标识符 (仅支持群组)
        text (str): 语音文本 (必须是自然语句，不包含任何特殊符号)

    Returns:
        bool: 操作是否成功
    """
    chat_type = context.get_chat_type(chat_key)
    chat_id = context.get_chat_id(chat_key)

    if chat_type != "group":
        core.logger.error(f"不支持 {chat_type} 类型")
        return False

    try:
        await core.get_bot().call_api(
            "send_group_ai_record",
            group_id=int(chat_id),
            character=config.AI_VOICE_CHARACTER,
            text=text,
        )
        core.logger.info(f"[{chat_key}] 生成 AI 语音完成 (内容: {text})")
    except Exception as e:
        core.logger.error(f"[{chat_key}] 生成 AI 语音失败: {e} | 如果协议端不支持该功能，请禁用此插件")
        return False
    else:
        return True


@plugin.mount_cleanup_method()
async def clean_up():
    """清理插件"""



import time
from typing import Optional

from httpx import AsyncClient
from pydantic import Field

from nekro_agent.api import core, message
from nekro_agent.api.plugin import ConfigBase, NekroPlugin, SandboxMethodType
from nekro_agent.api.schemas import AgentCtx

plugin = NekroPlugin(
    name="Google搜索工具",
    module_name="google_search",
    description="提供Google搜索功能，获取实时信息",
    version="0.1.0",
    author="KroMiose",
    url="https://github.com/KroMiose/nekro-agent",
)


@plugin.mount_config()
class GoogleSearchConfig(ConfigBase):
    """Google搜索配置"""

    API_KEY: str = Field(
        default="",
        title="Google搜索API密钥",
        description="Google 搜索 API 密钥 <a href='https://developers.google.com/custom-search/v1/introduction?hl=zh-cn' target='_blank'>获取地址</a>",
        json_schema_extra={"is_secret": True},
    )
    CX_KEY: str = Field(
        default="",
        title="Google搜索CX密钥",
        description="Google 搜索 CX 密钥 <a href='https://programmablesearchengine.google.com/controlpanel/all' target='_blank'>获取地址</a>",
        json_schema_extra={"is_secret": True},
    )
    THROTTLE_TIME: int = Field(
        default=10,
        title="搜索冷却时间(秒)",
        description="同一关键词在此时间内重复搜索将被阻止",
    )
    MAX_RESULTS: int = Field(
        default=5,
        title="最大结果数",
        description="返回的搜索结果数量",
    )


# 获取配置
config: GoogleSearchConfig = plugin.get_config(GoogleSearchConfig)

_last_keyword = None
_last_call_time = 0


@plugin.mount_sandbox_method(
    SandboxMethodType.AGENT,
    name="Google搜索",
    description="使用Google搜索获取实时信息",
)
async def google_search(_ctx: AgentCtx, keyword: str) -> str:
    """使用 Google 搜索获取实时信息

    Args:
        keyword (str): 搜索关键词
    """
    global _last_keyword, _last_call_time

    # 防止重复搜索和频繁调用
    if keyword == _last_keyword and time.time() - _last_call_time < config.THROTTLE_TIME:
        return "[错误] 禁止频繁搜索相同内容，结果无变化"

    proxy = core.config.DEFAULT_PROXY
    api_key = config.API_KEY
    cx_key = config.CX_KEY
    max_results = config.MAX_RESULTS

    if not api_key or not cx_key:
        return "[Google] 未配置 API Key 或 CX Key"

    if proxy and not proxy.startswith("http"):
        proxy = f"http://{proxy}"

    headers = {"User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36"}

    try:
        async with AsyncClient(proxies=proxy) as cli:
            response = (
                await cli.get(
                    "https://www.googleapis.com/customsearch/v1",
                    headers=headers,
                    params={"key": api_key, "cx": cx_key, "q": keyword},
                )
            ).json()
    except Exception as e:
        core.logger.exception("Google搜索失败")
        return f"[Google] 搜索失败: {e!s}"

    try:
        items = response["items"]
        results = "\n".join([f"[{item['title']}] {item['snippet']} - from: {item['link']}" for item in items[:max_results]])
    except:
        return f"[Google] 未找到关于'{keyword}'的信息"

    _last_keyword = keyword
    _last_call_time = time.time()

    return f"[Google Search Results]\n{results}\nAnalyze and synthesize the above search results to provide insights. DO NOT directly repeat the search results - integrate them into a thoughtful response."


@plugin.mount_cleanup_method()
async def clean_up():
    """清理插件"""
    global _last_keyword, _last_call_time
    _last_keyword = None
    _last_call_time = 0



from .db_chat_channel import DBChatChannel
from .db_chat_message import DBChatMessage
from .db_exec_code import DBExecCode
from .db_plugin_data import DBPluginData
from .db_preset import DBPreset
from .db_user import DBUser



import base64
from pathlib import Path
from typing import Any, Dict, List, Literal, Union

import magic

from .templates.base import PromptTemplate

mime = magic.Magic(mime=True)


class OpenAIChatMessage:
    """OpenAI 聊天消息"""

    def __init__(self, role: Literal["user", "assistant", "system"], content: List[Dict[str, Any]]):
        self.role: Literal["user", "assistant", "system"] = role
        self.content = content

    def to_dict(self) -> Dict[str, Any]:
        """转换为字典

        将连续的文本内容进行聚合，例如:
        [{"type": "text", "text": "你好"}, {"type": "text", "text": "世界"}]
        会被聚合为:
        [{"type": "text", "text": "你好世界"}]
        """

        if all(_c["type"] == "text" for _c in self.content):
            return {"role": self.role, "content": "".join(_c["text"] for _c in self.content)}

        merged_content: List[Dict[str, Any]] = []
        current_text = ""
        for segment in self.content:
            if segment["type"] == "text":
                current_text += segment["text"]
            else:
                if current_text:
                    merged_content.append({"type": "text", "text": current_text})
                    current_text = ""
                merged_content.append(segment)
        if current_text:
            merged_content.append({"type": "text", "text": current_text})
        return {"role": self.role, "content": merged_content}

    @classmethod
    def from_text(cls, role: Literal["user", "assistant", "system"], text: str) -> "OpenAIChatMessage":
        """从文本生成消息"""
        return cls(role, [ContentSegment.text_content(text)])

    @classmethod
    def from_template(cls, role: Literal["user", "assistant", "system"], template: PromptTemplate) -> "OpenAIChatMessage":
        """添加 Jinja2 模板渲染片段"""
        return cls(role, [ContentSegment.text_content(template.render())])

    @classmethod
    def create_empty(cls, role: Literal["user", "assistant", "system"]) -> "OpenAIChatMessage":
        """创建空消息"""
        return cls(role, [])

    def add(self, segment: Dict[str, Any]) -> "OpenAIChatMessage":
        """添加内容片段"""
        self.content.append(segment)
        return self

    def batch_add(self, segments: List[Dict[str, Any]]) -> "OpenAIChatMessage":
        """批量添加内容片段"""
        self.content.extend(segments)
        return self

    def extend(self, other: "OpenAIChatMessage") -> "OpenAIChatMessage":
        """合并消息"""
        if self.role != other.role:
            raise ValueError("消息角色不一致")
        return OpenAIChatMessage(self.role, self.content + other.content)

    def tidy(self) -> "OpenAIChatMessage":
        """整理消息合并所有连续的文本内容"""
        merged_content: List[Dict[str, Any]] = []
        current_text = ""
        for segment in self.content:
            if segment["type"] == "text":
                current_text += segment["text"]
            else:
                if current_text:
                    merged_content.append({"type": "text", "text": current_text})
                    current_text = ""
                merged_content.append(segment)
        if current_text:
            merged_content.append({"type": "text", "text": current_text})
        return OpenAIChatMessage(self.role, merged_content)


class ContentSegment:
    """内容片段生成器"""

    @staticmethod
    def image_content(image_url: str) -> Dict[str, Any]:
        """生成图片内容片段"""
        return {"type": "image_url", "image_url": {"url": image_url}}

    @staticmethod
    def image_content_from_bytes(image_bytes: bytes, image_type: str) -> Dict[str, Any]:
        """根据图片字节生成图片内容片段"""
        return {
            "type": "image_url",
            "image_url": {"url": f"data:image/{image_type};base64,{base64.b64encode(image_bytes).decode()}"},
        }

    @staticmethod
    def image_content_from_path(image_path: Union[str, Path]) -> Dict[str, Any]:
        """根据图片路径生成图片内容片段"""
        path = Path(image_path)
        if not path.exists():
            raise FileNotFoundError(f"图片路径不存在: {path}")

        file_bytes = path.read_bytes()
        mime_type = mime.from_buffer(file_bytes)
        mime_type = "image/png" if mime_type == "image/gif" else mime_type

        return {
            "type": "image_url",
            "image_url": {"url": f"data:{mime_type};base64,{base64.b64encode(file_bytes).decode()}"},
        }

    @staticmethod
    def text_content(text: str) -> Dict[str, Any]:
        """生成文本内容片段"""
        return {"type": "text", "text": text}



import json
from pathlib import Path
from typing import AsyncGenerator, Dict, List, Optional

from fastapi import APIRouter, Body, Depends, HTTPException, Request
from fastapi import Path as PathParam
from pydantic import BaseModel
from sse_starlette.sse import EventSourceResponse

from nekro_agent.core import logger
from nekro_agent.core.config import config
from nekro_agent.core.os_env import WORKDIR_PLUGIN_DIR
from nekro_agent.models.db_user import DBUser
from nekro_agent.schemas.message import Ret
from nekro_agent.services.plugin.collector import plugin_collector
from nekro_agent.services.plugin.generator import (
    apply_plugin_code,
    generate_plugin_code,
    generate_plugin_code_stream,
    generate_plugin_template,
)
from nekro_agent.services.user.deps import get_current_active_user
from nekro_agent.services.user.perm import Role, require_role

router = APIRouter(prefix="/plugin-editor", tags=["Plugin Editor"])


class FileContentResponse(BaseModel):
    """文件内容响应"""

    content: str


@router.get("/files", summary="获取插件文件列表")
@require_role(Role.Admin)
async def get_plugin_files(
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """获取插件文件列表"""
    try:
        if not WORKDIR_PLUGIN_DIR:
            return Ret.fail(msg="工作目录插件目录未配置")

        plugin_dir = Path(WORKDIR_PLUGIN_DIR)
        if not plugin_dir.exists():
            plugin_dir.mkdir(parents=True, exist_ok=True)
            return Ret.success(msg="获取成功", data=[])

        files = []
        for pattern in ["**/*.py", "**/*.py.disabled"]:
            for file in plugin_dir.glob(pattern):
                files.append(str(file.relative_to(plugin_dir)))

        return Ret.success(msg="获取成功", data=files)
    except Exception as e:
        logger.error(f"获取插件文件列表失败: {e}")
        return Ret.error(msg=f"获取失败: {e!s}")


@router.get("/file/{file_path:path}", summary="获取插件文件内容")
@require_role(Role.Admin)
async def get_plugin_file_content(
    file_path: str = PathParam(...),
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """获取插件文件内容"""
    try:
        if not WORKDIR_PLUGIN_DIR:
            return Ret.fail(msg="工作目录插件目录未配置")

        # 安全检查：确保路径在工作目录内
        plugin_dir = Path(WORKDIR_PLUGIN_DIR)
        full_path = plugin_dir / file_path

        if not full_path.exists():
            return Ret.fail(msg=f"文件 {file_path} 不存在")

        # 检查文件是否在插件目录内
        try:
            full_path.relative_to(plugin_dir)
        except ValueError:
            return Ret.fail(msg="文件路径非法")

        # 读取文件内容
        content = full_path.read_text(encoding="utf-8")
        return Ret.success(msg="获取成功", data={"content": content})
    except Exception as e:
        logger.error(f"获取插件文件内容失败: {e}")
        return Ret.error(msg=f"获取失败: {e!s}")


@router.post("/file/{file_path:path}", summary="保存插件文件")
@require_role(Role.Admin)
async def save_plugin_file(
    request: Request,
    file_path: str = PathParam(...),
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """保存插件文件"""
    try:
        if not WORKDIR_PLUGIN_DIR:
            return Ret.fail(msg="工作目录插件目录未配置")

        # 安全检查：确保路径在工作目录内
        plugin_dir = Path(WORKDIR_PLUGIN_DIR)
        full_path = plugin_dir / file_path

        # 检查文件是否在插件目录内
        try:
            full_path.relative_to(plugin_dir)
        except ValueError:
            return Ret.fail(msg="文件路径非法")

        # 确保目标目录存在
        full_path.parent.mkdir(parents=True, exist_ok=True)

        # 读取请求体
        content = await request.body()
        content_str = content.decode("utf-8")

        # 写入文件
        full_path.write_text(content_str, encoding="utf-8")
        return Ret.success(msg="保存成功")
    except Exception as e:
        logger.error(f"保存插件文件失败: {e}")
        return Ret.error(msg=f"保存失败: {e!s}")


@router.delete("/files/{file_path:path}", summary="删除插件文件")
@require_role(Role.Admin)
async def delete_plugin_file(
    file_path: str = PathParam(...),
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """删除插件文件"""
    try:
        if not WORKDIR_PLUGIN_DIR:
            return Ret.fail(msg="工作目录插件目录未配置")

        # 安全检查：确保路径在工作目录内
        plugin_dir = Path(WORKDIR_PLUGIN_DIR)
        full_path = plugin_dir / file_path

        # 检查文件是否在插件目录内
        try:
            full_path.relative_to(plugin_dir)
        except ValueError:
            return Ret.fail(msg="文件路径非法")

        # 检查文件是否存在
        if not full_path.exists():
            return Ret.fail(msg=f"文件 {file_path} 不存在")

        await plugin_collector.unload_plugin_by_module_name(full_path.stem)

        # 删除文件
        full_path.unlink()
        return Ret.success(msg="删除成功")
    except Exception as e:
        logger.error(f"删除插件文件失败: {e}")
        return Ret.error(msg=f"删除失败: {e!s}")


class GenerateCodeRequest(BaseModel):
    """生成代码请求体"""

    file_path: str
    prompt: str
    current_code: Optional[str] = None


class GenerateCodeResponse(BaseModel):
    """生成代码响应体"""

    code: str


@router.post("/generate", summary="生成插件代码")
@require_role(Role.Admin)
async def generate_code(
    body: GenerateCodeRequest,
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """生成插件代码"""
    try:
        if not body.prompt:
            return Ret.fail(msg="提示词不能为空")

        code = await generate_plugin_code(
            file_path=body.file_path,
            prompt=body.prompt,
            current_code=body.current_code,
        )
        return Ret.success(msg="生成成功", data={"code": code})
    except Exception as e:
        logger.error(f"生成插件代码失败: {e}")
        return Ret.error(msg=f"生成失败: {e!s}")


@router.post("/generate/stream", summary="流式生成插件代码")
@require_role(Role.Admin)
async def generate_code_stream(
    body: GenerateCodeRequest,
    _current_user: DBUser = Depends(get_current_active_user),
) -> EventSourceResponse:
    """流式生成插件代码"""

    async def event_generator() -> AsyncGenerator[str, None]:
        try:
            # 使用简化的流式生成器直接产生内容
            async for chunk in generate_plugin_code_stream(
                file_path=body.file_path,
                prompt=body.prompt,
                current_code=body.current_code,
            ):
                # 每个chunk都作为内容发送
                yield json.dumps({"type": "content", "content": chunk})

            # 生成完成后发送完成信号
            yield json.dumps({"type": "done"})
        except Exception as e:
            logger.error(f"流式生成插件代码失败: {e}")
            yield json.dumps({"type": "error", "error": str(e)})

    return EventSourceResponse(event_generator())


@router.post("/apply", summary="应用生成的代码")
@require_role(Role.Admin)
async def apply_code(
    body: GenerateCodeRequest,
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """应用生成的代码"""
    try:
        if not body.prompt or not body.current_code:
            return Ret.fail(msg="参数不完整")

        code = await apply_plugin_code(
            file_path=body.file_path,
            prompt=body.prompt,
            current_code=body.current_code,
        )
        return Ret.success(msg="应用成功", data={"code": code})
    except Exception as e:
        logger.error(f"应用生成的代码失败: {e}")
        return Ret.error(msg=f"应用失败: {e!s}")


class TemplateRequest(BaseModel):
    """模板请求体"""

    name: str
    description: str


class TemplateResponse(BaseModel):
    """模板响应体"""

    template: str


@router.post("/template", summary="生成插件模板")
@require_role(Role.Admin)
async def create_plugin_template(
    body: TemplateRequest,
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """生成插件模板"""
    try:
        template = generate_plugin_template(name=body.name, description=body.description)
        return Ret.success(msg="生成成功", data={"template": template})
    except Exception as e:
        logger.error(f"生成插件模板失败: {e}")
        return Ret.error(msg=f"生成失败: {e!s}")



import asyncio
import json
from datetime import datetime, timedelta
from typing import Dict, List, Optional, Union

from fastapi import APIRouter, Depends, Query
from fastapi.responses import StreamingResponse
from sse_starlette.sse import EventSourceResponse
from tortoise.functions import Avg, Count

from nekro_agent.models.db_chat_channel import DBChatChannel
from nekro_agent.models.db_chat_message import DBChatMessage
from nekro_agent.models.db_exec_code import DBExecCode, ExecStopType
from nekro_agent.models.db_user import DBUser
from nekro_agent.schemas.message import Ret
from nekro_agent.services.user.deps import get_current_active_user

router = APIRouter(prefix="/dashboard", tags=["Dashboard"])


async def get_time_range(time_range: str = "day") -> datetime:
    now = datetime.now()
    if time_range == "day":
        return now.replace(hour=0, minute=0, second=0, microsecond=0)
    if time_range == "week":
        start_time = now - timedelta(days=now.weekday())
        return start_time.replace(hour=0, minute=0, second=0, microsecond=0)
    if time_range == "month":
        return now.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
    return now - timedelta(days=1)


@router.get("/overview", summary="获取仪表盘概览数据")
async def get_dashboard_overview(
    time_range: str = "day",
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    start_time = await get_time_range(time_range)

    total_messages = await DBChatMessage.filter(create_time__gte=start_time).count()
    active_sessions = len(
        await DBChatMessage.filter(
            create_time__gte=start_time,
        )
        .distinct()
        .values_list("chat_key", flat=True),
    )
    unique_users = await DBChatMessage.filter(create_time__gte=start_time).distinct().values_list("sender_id", flat=True)
    total_sandbox_calls = await DBExecCode.filter(create_time__gte=start_time).count()
    success_calls = await DBExecCode.filter(create_time__gte=start_time, success=True).count()
    failed_calls = total_sandbox_calls - success_calls
    success_rate = round(success_calls / total_sandbox_calls * 100, 2) if total_sandbox_calls > 0 else 0

    return Ret.success(
        msg="获取成功",
        data={
            "total_messages": total_messages,
            "active_sessions": active_sessions,
            "unique_users": len(unique_users),
            "total_sandbox_calls": total_sandbox_calls,
            "success_calls": success_calls,
            "failed_calls": failed_calls,
            "success_rate": success_rate,
        },
    )


@router.get("/trends", summary="获取趋势数据")
async def get_trends(
    metrics: str,
    time_range: str = "day",
    interval: str = "hour",
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    """获取趋势数据"""
    # 计算时间范围
    now = datetime.now()
    if time_range == "day":
        start_time = now.replace(hour=0, minute=0, second=0, microsecond=0)
        if interval == "hour":
            intervals = 24
            delta = timedelta(hours=1)
        else:
            intervals = 24
            delta = timedelta(hours=1)
    elif time_range == "week":
        start_time = now - timedelta(days=now.weekday())
        start_time = start_time.replace(hour=0, minute=0, second=0, microsecond=0)
        if interval == "day":
            intervals = 7
            delta = timedelta(days=1)
        else:
            intervals = 7 * 24
            delta = timedelta(hours=1)
    elif time_range == "month":
        start_time = now.replace(day=1, hour=0, minute=0, second=0, microsecond=0)
        if interval == "day":
            intervals = 30
            delta = timedelta(days=1)
        else:
            intervals = 30
            delta = timedelta(days=1)
    else:
        start_time = now - timedelta(days=1)
        intervals = 24
        delta = timedelta(hours=1)

    # 解析请求的指标
    metrics_list = metrics.split(",")

    # 准备结果数据
    result = []
    current_time = start_time

    for _ in range(intervals):
        next_time = current_time + delta
        data_point: Dict[str, Union[str, int, float]] = {"timestamp": current_time.isoformat()}

        # 查询各指标数据
        if "messages" in metrics_list:
            messages_count = await DBChatMessage.filter(create_time__gte=current_time, create_time__lt=next_time).count()
            data_point["messages"] = messages_count

        if any(m in metrics_list for m in ["sandbox_calls", "success_calls", "failed_calls", "success_rate"]):
            sandbox_calls = await DBExecCode.filter(create_time__gte=current_time, create_time__lt=next_time).count()

            if "sandbox_calls" in metrics_list:
                data_point["sandbox_calls"] = sandbox_calls

            if any(m in metrics_list for m in ["success_calls", "failed_calls", "success_rate"]):
                success_calls = await DBExecCode.filter(
                    create_time__gte=current_time,
                    create_time__lt=next_time,
                    success=True,
                ).count()

                if "success_calls" in metrics_list:
                    data_point["success_calls"] = success_calls

                if "failed_calls" in metrics_list:
                    data_point["failed_calls"] = sandbox_calls - success_calls

                if "success_rate" in metrics_list:
                    data_point["success_rate"] = round(success_calls / sandbox_calls * 100, 2) if sandbox_calls > 0 else 0

        result.append(data_point)
        current_time = next_time

    return Ret.success(
        msg="获取成功",
        data=result,
    )


@router.get("/ranking", summary="获取排名数据")
async def get_ranking(
    ranking_type: str,
    time_range: str = "day",
    limit: int = 10,
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    start_time = await get_time_range(time_range)

    if ranking_type == "users":
        execs = await DBExecCode.filter(
            create_time__gte=start_time,
            trigger_user_id__not_in=[0, -1],  # 过滤掉系统触发的执行
        ).all()

        user_counts = {}
        for _exec in execs:
            user_id = _exec.trigger_user_id
            user_name = _exec.trigger_user_name

            if user_id not in user_counts:
                user_counts[user_id] = {
                    "id": user_id,
                    "name": user_name,
                    "value": 0,
                }

            user_counts[user_id]["value"] += 1

        result = sorted(user_counts.values(), key=lambda x: x["value"], reverse=True)[:limit]

        return Ret.success(
            msg="获取成功",
            data=result,
        )

    return Ret.success(
        msg="获取成功",
        data=[],
    )


@router.get("/stats/stream", summary="获取实时统计数据流")
async def get_stats_stream(
    granularity: int = Query(10, description="数据粒度（分钟）", ge=1, le=60),
    _current_user: DBUser = Depends(get_current_active_user),
):
    async def generate():
        try:
            # 初始化计数器
            last_message_count = await DBChatMessage.all().count()
            last_sandbox_count = await DBExecCode.all().count()
            last_success_count = await DBExecCode.filter(success=True).all().count()

            # 计算开始时间（当前时间减去50个粒度单位）
            start_time = datetime.now() - timedelta(minutes=granularity * 50)

            # 对齐到粒度的整点时间
            current_time = start_time.replace(
                minute=(start_time.minute // granularity) * granularity,
                second=0,
                microsecond=0,
            )

            # 发送历史数据（最多50个数据点）
            while current_time < datetime.now():
                next_time = current_time + timedelta(minutes=granularity)

                # 查询该时间段内的数据
                messages = await DBChatMessage.filter(
                    create_time__gte=current_time,
                    create_time__lt=next_time,
                ).count()

                sandbox_calls = await DBExecCode.filter(
                    create_time__gte=current_time,
                    create_time__lt=next_time,
                ).count()

                success_calls = await DBExecCode.filter(
                    create_time__gte=current_time,
                    create_time__lt=next_time,
                    success=True,
                ).count()

                execs = await DBExecCode.filter(
                    create_time__gte=current_time,
                    create_time__lt=next_time,
                ).all()

                avg_exec_time = sum(_exec.exec_time_ms for _exec in execs) / len(execs) if execs else 0

                # 发送数据（即使没有活动也发送，以保持图表连续性）
                yield json.dumps(
                    {
                        "timestamp": current_time.isoformat(),
                        "recent_messages": messages,
                        "recent_sandbox_calls": sandbox_calls,
                        "recent_success_calls": success_calls,
                        "recent_avg_exec_time": round(avg_exec_time, 2),
                    },
                )
                await asyncio.sleep(0.01)  # 短暂延迟，避免一次性发送太多数据

                current_time = next_time

            # 计算下一个对齐的时间点
            now = datetime.now()
            next_aligned_time = now.replace(
                minute=((now.minute // granularity) * granularity + granularity) % 60,
                second=0,
                microsecond=0,
            )
            # 如果下一个对齐时间的分钟小于当前分钟，说明跨小时了
            if next_aligned_time.minute < now.minute:
                next_aligned_time = next_aligned_time.replace(hour=next_aligned_time.hour + 1)

            # 实时监控循环
            while True:
                # 等待到下一个对齐的时间点
                wait_seconds = (next_aligned_time - datetime.now()).total_seconds()
                if wait_seconds > 0:
                    await asyncio.sleep(wait_seconds)

                # 查询从上次发送到现在的数据
                current_message_count = await DBChatMessage.all().count()
                current_sandbox_count = await DBExecCode.all().count()
                current_success_count = await DBExecCode.filter(success=True).all().count()

                recent_messages = current_message_count - last_message_count
                recent_sandbox_calls = current_sandbox_count - last_sandbox_count
                recent_success_calls = current_success_count - last_success_count

                # 查询最近一个粒度时间内的平均执行时间
                recent_execs = await DBExecCode.filter(
                    create_time__gte=datetime.now() - timedelta(minutes=granularity),
                ).all()
                recent_avg_exec_time = (
                    sum(_exec.exec_time_ms for _exec in recent_execs) / len(recent_execs) if recent_execs else 0
                )

                # 更新计数器
                last_message_count = current_message_count
                last_sandbox_count = current_sandbox_count
                last_success_count = current_success_count

                # 发送数据
                yield json.dumps(
                    {
                        "timestamp": next_aligned_time.isoformat(),
                        "recent_messages": recent_messages,
                        "recent_sandbox_calls": recent_sandbox_calls,
                        "recent_success_calls": recent_success_calls,
                        "recent_avg_exec_time": round(recent_avg_exec_time, 2),
                    },
                )

                # 计算下一个对齐的时间点
                next_aligned_time = next_aligned_time + timedelta(minutes=granularity)

        except Exception as e:
            print(f"Stream error: {e}")
            yield json.dumps({"error": str(e)})

    return EventSourceResponse(generate())


@router.get("/distributions", summary="获取所有分布数据")
async def get_distributions(
    time_range: str = "day",
    _current_user: DBUser = Depends(get_current_active_user),
) -> Ret:
    start_time = await get_time_range(time_range)

    total_execs = await DBExecCode.filter(create_time__gte=start_time).count()
    stop_type_data = []

    if total_execs > 0:
        for stop_type in ExecStopType:
            count = await DBExecCode.filter(create_time__gte=start_time, stop_type=stop_type).count()
            if count > 0:
                stop_type_data.append(
                    {
                        "label": stop_type.value,
                        "value": count,
                        "percentage": round(count / total_execs * 100, 2),
                    },
                )

    total_messages = await DBChatMessage.filter(create_time__gte=start_time).count()
    message_type_data = []

    if total_messages > 0:
        group_count = await DBChatMessage.filter(create_time__gte=start_time, chat_type="group").count()
        private_count = total_messages - group_count

        message_type_data = [
            {
                "label": "群聊消息",
                "value": group_count,
                "percentage": round(group_count / total_messages * 100, 2),
            },
            {
                "label": "私聊消息",
                "value": private_count,
                "percentage": round(private_count / total_messages * 100, 2),
            },
        ]

    return Ret.success(
        msg="获取成功",
        data={
            "stop_type": stop_type_data,
            "message_type": message_type_data,
        },
    )



"""插件管理服务

提供插件的管理相关功能，如获取插件列表、配置管理等。
"""

import importlib
import inspect
import json
import sys
from pathlib import Path
from typing import Any, Dict, List, Optional, Tuple, Union

from nekro_agent.core import logger
from nekro_agent.core.config import CONFIG_PATH, config
from nekro_agent.core.core_utils import ConfigBase
from nekro_agent.services.config_service import ConfigService
from nekro_agent.services.plugin.collector import plugin_collector
from nekro_agent.services.plugin.schema import SandboxMethodType


async def get_all_ext_meta_data() -> List[dict]:
    """获取所有已注册插件的元数据"""
    plugins = plugin_collector.get_all_plugins()
    return [
        {
            "id": plugin.key,
            "name": plugin.name,
            "description": plugin.description,
            "version": plugin.version,
            "author": plugin.author,
            "enabled": plugin.is_enabled,
            "hasConfig": hasattr(plugin, "_Configs") and plugin._Configs != ConfigBase,  # noqa: SLF001
            "url": plugin.url or "",
            "isBuiltin": plugin.is_builtin,
            "isPackage": plugin.is_package,
        }
        for plugin in plugins
    ]


async def get_plugin_detail(plugin_id: str) -> Optional[dict]:
    """获取指定插件的详细信息"""
    plugin = plugin_collector.get_plugin(plugin_id)
    if not plugin:
        return None

    # 获取方法信息
    methods = [
        {
            "name": method.name,
            "type": method.method_type.value,
            "description": method.description or "",
        }
        for method in plugin.sandbox_methods
    ]

    # 获取 webhook 信息
    webhooks = [
        {
            "endpoint": endpoint,
            "name": method.name,
            "description": method.description or "",
        }
        for endpoint, method in plugin.webhook_methods.items()
    ]

    # 构建插件详情
    return {
        "name": plugin.name,
        "moduleName": plugin.module_name,
        "id": plugin.key,
        "version": plugin.version,
        "description": plugin.description,
        "author": plugin.author,
        "url": plugin.url or "",
        "enabled": plugin.is_enabled,
        "hasConfig": hasattr(plugin, "_Configs") and plugin._Configs != ConfigBase,  # noqa: SLF001
        "methods": methods,
        "webhooks": webhooks,
        "isBuiltin": plugin.is_builtin,
        "isPackage": plugin.is_package,
    }


async def enable_plugin(plugin_id: str) -> bool:
    """启用插件"""
    plugin = plugin_collector.get_plugin(plugin_id)
    if not plugin:
        return False

    if plugin.is_enabled:
        return True  # 已经启用，直接返回成功

    try:
        plugin.enable()
        if plugin.key not in config.PLUGIN_ENABLED:
            config.PLUGIN_ENABLED.append(plugin.key)
            ConfigService.save_config(config, CONFIG_PATH)
    except Exception as e:
        logger.error(f"启用插件失败: {plugin_id}, 错误: {e}")
        return False
    else:
        return True


async def disable_plugin(plugin_id: str) -> bool:
    """禁用插件"""
    plugin = plugin_collector.get_plugin(plugin_id)
    if not plugin:
        return False

    if not plugin.is_enabled:
        return True  # 已经禁用，直接返回成功

    try:
        plugin.disable()
        if plugin.key in config.PLUGIN_ENABLED:
            config.PLUGIN_ENABLED.remove(plugin.key)
            ConfigService.save_config(config, CONFIG_PATH)
    except Exception as e:
        logger.error(f"禁用插件失败: {plugin_id}, 错误: {e}")
        return False
    else:
        return True


async def get_plugin_config(plugin_id: str) -> Optional[List[Dict[str, Any]]]:
    """获取插件配置列表"""
    plugin = plugin_collector.get_plugin(plugin_id)
    if not plugin or not hasattr(plugin, "_Configs") or plugin._Configs == ConfigBase:  # noqa: SLF001
        return None

    try:
        config = plugin.get_config()
        if not config:
            return None

        # 使用配置服务获取配置列表
        return ConfigService.get_config_list(config)
    except Exception as e:
        logger.error(f"获取插件配置失败: {plugin_id}, 错误: {e}")
        return None


async def save_plugin_config(plugin_id: str, configs: Dict[str, str]) -> Tuple[bool, Optional[str]]:
    """保存插件配置

    Args:
        plugin_id: 插件ID
        configs: 配置项字典，键为配置项名称，值为字符串形式的配置值

    Returns:
        (成功状态, 错误信息)
    """
    plugin = plugin_collector.get_plugin(plugin_id)
    if not plugin or not hasattr(plugin, "_Configs") or plugin._Configs == ConfigBase:  # noqa: SLF001
        return False, f"插件 {plugin_id} 不存在或无配置"

    try:
        config = plugin.get_config()
        if not config:
            return False, f"插件 {plugin_id} 配置获取失败"

        # 使用配置服务批量更新配置
        success, error_msg = ConfigService.batch_update_config(config, configs)
        if not success:
            return False, error_msg

        # 保存配置
        success, error_msg = ConfigService.save_config(config, plugin._plugin_config_path)  # noqa: SLF001
        if not success:
            return False, f"保存配置失败: {error_msg}"

    except Exception as e:
        logger.error(f"保存插件配置失败: {plugin_id}, 错误: {e}")
        return False, f"保存失败: {e!s}"
    else:
        return True, None



from pathlib import Path
from typing import List

from pydantic import Field

from nekro_agent.api import core
from nekro_agent.api.plugin import ConfigBase, NekroPlugin, SandboxMethodType
from nekro_agent.api.schemas import AgentCtx
from nekro_agent.core.config import ModelConfigGroup
from nekro_agent.core.config import config as core_config
from nekro_agent.services.agent.creator import ContentSegment, OpenAIChatMessage
from nekro_agent.services.agent.openai import gen_openai_chat_response
from nekro_agent.tools.path_convertor import convert_to_host_path

plugin = NekroPlugin(
    name="图片查看插件",
    module_name="view_image",
    description="提供图片查看功能",
    version="0.1.0",
    author="KroMiose",
    url="https://github.com/KroMiose/nekro-agent",
)


@plugin.mount_config()
class ViewImageConfig(ConfigBase):
    """基础配置"""

    VISION_MODEL: str = Field(
        default="default",
        title="用于查看图片的模型,如果主模型自带视觉请勿开启本插件",
        description="用于查看图片并描述内容,以文本形式返回给主模型",
        json_schema_extra={"ref_model_groups": True, "required": True, "model_type": "chat"},
    )


def get_view_image_config() -> ViewImageConfig:
    """获取最新的记忆模块配置"""
    return plugin.get_config(ViewImageConfig)


# 根据模型名获取模型组配置项
def get_model_group_info(model_name: str) -> ModelConfigGroup:
    try:
        return core_config.MODEL_GROUPS[model_name]
    except KeyError as e:
        raise ValueError(f"模型组 '{model_name}' 不存在，请确认配置正确") from e


@plugin.mount_sandbox_method(SandboxMethodType.AGENT, "图片观察工具")
async def view_image(_ctx: AgentCtx, images: List[str]):
    """利用视觉观察图片

    Attention: Do not repeatedly observe the contents of the visual image given in the context!

    Args:
        images (List[str]): 图片共享路径或在线url列表
    """

    core.logger.debug(f"图片观察工具: {images}")

    # 获取插件配置
    cfg: ViewImageConfig = get_view_image_config()
    vision_model_name = cfg.VISION_MODEL

    if not vision_model_name:
        raise ValueError("未配置有效的视觉模型 (VISION_MODEL)，无法使用图片观察工具。")

    vision_model_group = get_model_group_info(vision_model_name)

    if vision_model_group.MODEL_TYPE != "chat":
        raise ValueError("请选择正确的模型组")

    if not vision_model_group.ENABLE_VISION:
        raise ValueError("当前模型组不支持视觉功能")

    # 构造发送给视觉模型的消息
    vision_prompt = "Describe the content shown in this images in relatively detailed language."
    vision_msg = OpenAIChatMessage.from_text("system", vision_prompt)

    for i, image_path in enumerate(images):
        # 判断是否为URL（简单判断是否以http开头）
        if image_path.startswith(("http://", "https://")):
            # 使用URL方式
            vision_msg.batch_add(
                [
                    ContentSegment.text_content(f"Image {i+1}: {image_path}"),
                    ContentSegment.image_content(image_path),
                ],
            )
        else:
            # 使用文件路径方式
            path = convert_to_host_path(Path(image_path), chat_key=_ctx.from_chat_key)
            if not path.exists():
                raise ValueError(f"图片路径不存在: {image_path}")
            vision_msg.batch_add(
                [
                    ContentSegment.text_content(f"Image {i+1}: {image_path}"),
                    ContentSegment.image_content_from_path(path),
                ],
            )

    # 调用 OpenAI 服务获取图片描述
    try:
        description_response = await gen_openai_chat_response(
            model=vision_model_group.CHAT_MODEL,
            messages=[vision_msg.to_dict()],
            base_url=vision_model_group.BASE_URL,
            api_key=vision_model_group.API_KEY,
        )
        # 响应对象现在是 OpenAIResponse
        description = description_response.response_content

        if not description:
            return ""

    except Exception as e:
        core.logger.error(f"调用视觉模型 ({vision_model_group.CHAT_MODEL}) 时出错: {e}")
        raise RuntimeError(f"调用视觉模型失败: {e}") from e

    return f"The visual description of the images is as follows:\n{description}"



from typing import List, Optional

from pydantic import BaseModel, Field

from nekro_agent.systems.cloud.schemas.base import BasicResponse


class StarCheckData(BaseModel):
    """Star 检查数据结构"""

    starred_repositories: List[str] = Field(default_factory=list, alias="starredRepositories")
    unstarred_repositories: List[str] = Field(default_factory=list, alias="unstarredRepositories")
    all_starred: bool = Field(default=False, alias="allStarred")


class StarCheckResponse(BasicResponse):
    """官方仓库 Star 检查响应"""

    data: Optional[StarCheckData] = None

    @classmethod
    def process_exception(cls, exc: Exception):
        return cls(
            success=False,
            message=f"检查 GitHub Star 状态失败: {exc!s}",
            data=None,
        )



from nekro_agent.core.logger import logger
from nekro_agent.systems.cloud.schemas.auth import StarCheckResponse

from .client import get_client


async def check_official_repos_starred() -> StarCheckResponse:
    """检查用户是否已给官方GitHub仓库点亮Star

    返回用户已Star和未Star的仓库列表，以及是否已Star所有指定仓库

    Returns:
        StarCheckResponse: 包含Star状态的响应结果
    """
    try:
        async with get_client(require_auth=True) as client:
            response = await client.get(
                url="/api/auth/official-repos-starred",
            )
            response.raise_for_status()
            return StarCheckResponse(**response.json())
    except Exception as e:
        logger.error(f"检查GitHub仓库Star状态发生错误: {e}")
        return StarCheckResponse.process_exception(e)



from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional

from httpx import HTTPStatusError
from pydantic import BaseModel, Field

from nekro_agent.systems.cloud.exceptions import NekroCloudDisabled


class BasicResponse(BaseModel):
    """基本响应"""

    success: bool = Field(..., description="是否成功")
    message: str = Field(default="", description="消息")
    error: Optional[str] = Field(default=None, description="错误信息")

    @classmethod
    def process_exception(cls, e: Exception):
        if isinstance(e, NekroCloudDisabled):
            return cls(
                success=False,
                message="Nekro AI 社区服务遥测未启用，当前实例暂无权限使用",
                error="Nekro AI 社区服务遥测未启用，当前实例暂无权限使用",
            )
        if isinstance(e, HTTPStatusError) and e.response.status_code in [401, 403]:
            print(f"111 {e.response.text} {e.response.status_code}")
            return cls(
                success=False,
                message="Nekro AI 社区 API Key 无效，请前往 Nekro AI 社区获取并配置",
                error=e.response.text,
            )
        raise e

    class Config:
        extra = "ignore"



from datetime import datetime
from typing import Optional
from zoneinfo import ZoneInfo

from tortoise import fields
from tortoise.models import Model


class DBUser(Model):
    """数据库用户模型"""

    id = fields.IntField(pk=True, generated=True, description="用户ID")
    username = fields.CharField(max_length=128, description="用户名")
    password = fields.CharField(max_length=128, description="密码")
    bind_qq = fields.CharField(max_length=32, unique=True, description="绑定的QQ号")

    perm_level = fields.IntField(description="权限等级")
    login_time = fields.DatetimeField(description="上次登录时间")

    ban_until = fields.DatetimeField(null=True, description="封禁截止时间")
    prevent_trigger_until = fields.DatetimeField(null=True, description="禁止触发截止时间")
    ext_data = fields.JSONField(default=dict, description="扩展数据")

    create_time = fields.DatetimeField(auto_now_add=True, description="创建时间")
    update_time = fields.DatetimeField(auto_now=True, description="更新时间")

    @property
    def is_active(self) -> bool:
        """检查用户是否处于活跃状态（未被封禁）"""
        now = datetime.now(ZoneInfo("Asia/Shanghai"))
        if self.ban_until is None:
            return True
        # 确保 ban_until 也是带时区的，如果是 UTC 时间则转换为上海时间
        ban_until = self.ban_until
        if ban_until.tzinfo is None:
            ban_until = ban_until.replace(tzinfo=ZoneInfo("UTC"))
        ban_until = ban_until.astimezone(ZoneInfo("Asia/Shanghai"))
        return now > ban_until  # 检查是否在封禁期

    @property
    def is_prevent_trigger(self) -> bool:
        """检查用户是否被禁止触发（临时或永久）"""
        now = datetime.now(ZoneInfo("Asia/Shanghai"))
        if self.prevent_trigger_until is None:
            return False
        # 确保 prevent_trigger_until 也是带时区的，如果是 UTC 时间则转换为上海时间
        prevent_until = self.prevent_trigger_until
        if prevent_until.tzinfo is None:
            prevent_until = prevent_until.replace(tzinfo=ZoneInfo("UTC"))
        prevent_until = prevent_until.astimezone(ZoneInfo("Asia/Shanghai"))
        return now < prevent_until  # 检查是否在禁止触发期间（现在时间小于结束时间）

    class Meta:  # type: ignore
        table = "user"



from datetime import datetime
from typing import Dict, List, Optional, Set

from pydantic import BaseModel, Field


class RepoSubscription(BaseModel):
    """仓库订阅信息"""

    repo_name: str
    events: Set[str] = Field(default_factory=set)  # 订阅的事件类型，如 "push", "issues", "issue_comment" 等
    created_at: float
    last_updated: float

    @classmethod
    def create(cls, repo_name: str, events: Optional[List[str]] = None):
        """创建仓库订阅信息"""
        now = datetime.now().timestamp()
        # 默认支持的事件类型更多了
        default_events = ["push", "issues", "pull_request", "release", "star"]
        return cls(
            repo_name=repo_name,
            events=set(events or default_events),  # 默认订阅更多类型的事件
            created_at=now,
            last_updated=now,
        )

    def update_events(self, events: List[str]):
        """更新订阅事件"""
        self.events = set(events)
        self.last_updated = datetime.now().timestamp()

    def add_event(self, event: str):
        """添加订阅事件"""
        self.events.add(event)
        self.last_updated = datetime.now().timestamp()

    def remove_event(self, event: str):
        """移除订阅事件"""
        if event in self.events:
            self.events.remove(event)
            self.last_updated = datetime.now().timestamp()

    def render(self) -> str:
        """渲染订阅信息"""
        events_str = ", ".join(sorted(self.events))
        created_time = datetime.fromtimestamp(self.created_at).strftime("%Y-%m-%d %H:%M:%S")
        return f"* {self.repo_name} (订阅事件: {events_str}, 创建时间: {created_time})"


class ChatSubscriptions(BaseModel):
    """会话订阅数据"""

    subscriptions: Dict[str, RepoSubscription] = Field(default_factory=dict)

    class Config:
        extra = "ignore"

    async def subscribe_repo(self, repo_name: str, events: Optional[List[str]] = None) -> RepoSubscription:
        """订阅仓库"""
        if repo_name in self.subscriptions:
            if events:
                self.subscriptions[repo_name].update_events(events)
            return self.subscriptions[repo_name]

        sub = RepoSubscription.create(repo_name, events)
        self.subscriptions[repo_name] = sub
        return sub

    async def unsubscribe_repo(self, repo_name: str) -> bool:
        """取消订阅仓库"""
        if repo_name in self.subscriptions:
            del self.subscriptions[repo_name]
            return True
        return False

    def get_subscription(self, repo_name: str) -> Optional[RepoSubscription]:
        """获取仓库订阅信息"""
        return self.subscriptions.get(repo_name)

    def is_subscribed(self, repo_name: str, event: Optional[str] = None) -> bool:
        """检查是否订阅了仓库的指定事件"""
        if repo_name not in self.subscriptions:
            return False
        if event is None:
            return True
        return event in self.subscriptions[repo_name].events

    def render_prompts(self) -> str:
        """渲染提示词"""
        if not self.subscriptions:
            return "当前没有GitHub仓库订阅 (使用 `subscribe_github_repo` 添加订阅)"

        subscriptions_str = "\n".join([sub.render() for sub in self.subscriptions.values()])
        return f"GitHub订阅仓库列表:\n{subscriptions_str}"
